/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// must be first: 
//#include "precompiled.h"

#include <Base/Common/XMLBase.hh>

using namespace std;
using namespace BIAS;

int XMLBase::XMLAdd(const xmlNodePtr Node, XMLIO& XMLObject) const {
  xmlNodePtr theNode;
  string TopLevelName;
  double Version = 0.0;
  XMLGetClassName(TopLevelName, Version);
  theNode = XMLObject.addChildNode(Node, TopLevelName);
  if (Version > 0.0) {
    XMLObject.addAttribute(theNode, "Version", Version);
  }
  XMLOut(theNode, XMLObject);
  return 0;
}


int XMLBase::XMLWrite(const std::string& Filename, 
                      int CompressionLevel,
                      bool AutoAddCompressionSuffix) const {
  XMLIO myXML;
  string TopLevelName; 
  double Version;
  XMLGetClassName(TopLevelName, Version);
  xmlNodePtr rootNode;
  rootNode = myXML.create(TopLevelName); 
  myXML.addAttribute(rootNode, "Version", Version);
  XMLOut(rootNode, myXML);
  myXML.SetCompression(CompressionLevel);
  int res = myXML.write(Filename, AutoAddCompressionSuffix); 
  myXML.clear();
  return res;
}


int XMLBase::XMLWriteToString(string &str) const
{
  XMLIO myXML;
  string TopLevelName; 
  double Version;
  XMLGetClassName(TopLevelName, Version);
  xmlNodePtr rootNode;
  rootNode = myXML.create(TopLevelName); 
  myXML.addAttribute(rootNode, "Version", Version);
  XMLOut(rootNode, myXML);
  myXML.WriteToString(str); 
  myXML.clear();
  return 0;
}

int XMLBase::XMLRead(const std::string& Filename) {
  int thereturn = 0;
  XMLIO myXML;
  xmlNodePtr rootNode;
  string TopLevelName;
  double Version;
  XMLGetClassName(TopLevelName, Version);
  rootNode = myXML.read(Filename);
  if (myXML.getNodeName(rootNode) != TopLevelName) {
#ifdef BIAS_DENUG
    BIASERR("No XML-File of type "<<TopLevelName<<", type is "
            <<myXML.getNodeName(rootNode) );
#endif
    //BIASBREAK; // no err because we will try to read projection from rig.
    thereturn = -1;
  } else if ((myXML.getAttributeValueDouble(rootNode, "Version") != Version) &&
             Version>0.0) {
    BIASERR("Wrong XML-File version (is "
            <<myXML.getAttributeValueDouble(rootNode, "Version") 
            <<"), but I expected "<<Version);
    BIASBREAK;
    thereturn = -2;
  } else thereturn = XMLIn(rootNode, myXML);
  myXML.clear();
  return thereturn;
}

int XMLBase::XMLReadFromString(const std::string& str) {
  int thereturn = 0;
  XMLIO myXML;
  xmlNodePtr rootNode;
  string TopLevelName;
  double Version;
  XMLGetClassName(TopLevelName, Version);
  rootNode = myXML.ReadFromString(str);
  if (myXML.getNodeName(rootNode) != TopLevelName) {
    BIASERR("No XML-File of type "<<TopLevelName<<", type is "
            <<myXML.getNodeName(rootNode) );
    thereturn = -1;
  } else if (myXML.getAttributeValueDouble(rootNode, "Version") != Version) {
    BIASERR("Wrong XML-File version (is "
            <<myXML.getAttributeValueDouble(rootNode, "Version") 
            <<"), but I expected "<<Version);
    thereturn = -2;
  } else thereturn = XMLIn(rootNode, myXML);
  myXML.clear();
  return thereturn;
}

