/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __AFFINETRANSFORMATION_HH__
#define __AFFINETRANSFORMATION_HH__
#include "bias_config.h" 

#include <Base/Math/Matrix3x4.hh>
#include <Base/Math/Matrix3x3.hh>
#include "HomgPoint3D.hh"
#include "RMatrixBase.hh"

namespace BIAS{
  /** @class AffineTransf
      @ingroup g_geometry
      last line of an affine transformation in 3d space is always 0 0 0 1
      and can therefor be droped from this class
      for this reason it is necessairy to overload certain operators 
      and functions e.g. *, Mult, ...
  */ 
  class BIASGeometryBase_EXPORT AffineTransf : public Matrix3x4<double>
  {
  public:
    AffineTransf();

    ~AffineTransf();

    inline RMatrixBase GetR() const
    { return R_; };

    inline Vector3<double> GetC() const
    { return C_; };

    HomgPoint3D operator*(HomgPoint3D& argv);

    /** args must be of type Vector4<double> because this should work 
        not only for HomgPoint3D but  also for HomPLane3D, ... */
    void Mult(Vector4<double>& argv, Vector4<double>& destv);
    
    AffineTransf& operator=(const AffineTransf& arg);

    void SetAsRotationMatrix(Matrix3x3<double>& R);

  protected:
    RMatrixBase R_;
    Vector3<double> C_;
  };

} // namespace BIAS
#endif // __AFFINETRANSFORMATION_HH__
