/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <iostream>

#include <Base/Image/ImageBase.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>

using namespace BIAS;
using namespace std;

int main(int argc, char *argv[])
{
  Image<unsigned char> im;

  
  if (argc<=1 || ImageIO::Load(argv[1], im)!=0){
    cerr << argv[0] << " <image>\n";
    im.Init(640, 480, 1);
    im.FillImageWithXValue();
  }

  int w=im.GetWidth();
  int h=im.GetHeight();
  im.SetROICorners(w>>2, h>>2, w-(w>>2), h-(h>>2));

  cout << "zero image outside of ("<<(w>>2)<<", "<<(h>>2)<<") <--> ("
       <<(w-(w>>2))<<", "<<(h-(h>>2))<<")\n";
  
  im.SetOutsideROIZero();

  ImageIO::Write("zeroed.mip", im);

  if (w>3 && h>3){
    for (int x=w-3; x<w; x++)
      for (int y=h-3; y<h; y++)
          // TODO 
      im.GetROI()->GetMaskDataArray()[y][x]=UCHAR_MAX;

    im.GetROI()->SetMaskValidFlag(true);
  } else {
    cerr << "image too small\n";
  } 

  im.GetROI()->Mask2Vector();
    
  // JW: do not display, too much output...
  //cout << "im: "<<*im.GetROI() << endl;

  ImageIO::Write("roi.mip", im);
  
  Image<unsigned char> im2;
  if (ImageIO::Read("roi.mip", im2)!=0){
    BIASERR("error loading image \"roi.mip\"");
    return -1;
  }
  
  // JW: do not display, too much output...
  //cout << "im2: "<<*im2.GetROI()<<endl;

  vector<Position> vec;
  vec.resize(3);

  vec[0]=Position(w>>1, h>>1);
  vec[1]=Position(0, 0);
  vec[2]=Position(w>>1, 0);

  im2.GetROI()->SetVector(vec);

  im2.GetROI()->Vector2Mask();

  cout << *im2.GetROI()<<endl;
  
  if (ImageIO::Read("zeroed.mip", im2)!=0){
    BIASERR("error reading image \"zeroed.mip\"");
    return -1;
  }
    
  return 0;
}
