#ifndef _TAG_H_INCLUDED_
#define _TAG_H_INCLUDED_


#include "bias_config.h" 
#include <string>
#include <iostream>
#include <Base/Debug/DebugSimple.hh>
#include <Base/Debug/Error.hh>


// Begin namespace BIAS
namespace BIAS {
  
  /** @class ExifTag
      @brief helper class containing EXIF Tag info and its's value.
      Wriiten e.g. in Jpeg header by digital cameras.
      @author Jan Woetzel 10/2004 */
  class ExifTag {
  public:
    ~ExifTag(){};
    ExifTag(){};
    ExifTag(const unsigned short _id,
	    const std::string _name,
	    const std::string _value)
      : id(_id), name(_name), value(_value)
    {};

    ExifTag(const unsigned short _id,
	    const std::string _name)
      : id(_id), name(_name), value("")
    {};
    
    std::ostream& Print(std::ostream& os=std::cout) const;

    /** @return true if the string contains a useful value, in particular not empty, not unknown etc.  JW */
    bool GoodValue(const std::string &val) const;
    
  
    // --- data members ---
    
    unsigned short id; // tag ID
    std::string name;  // name=description
    std::string value; // value of tag
  };
} // end namespace BIAS


// ------------------

/** @brief output operator for TagInfo
    @author Jan Woetzel */
std::ostream& operator<<(std::ostream& os, 
			 const BIAS::ExifTag &tag);


#endif
