/// @example ExampleLoadBOG.cpp
/// Shows how to read the BOG format which is used by DaimlerChrysler.
/// @author Jan Woetzel

// STD
#include <Base/Common/BIASpragma.hh>

#include <iostream>
#include <fstream>
#include <string>
#include <vector>


// BIAS
#include <Base/Math/Vector3.hh>
#include <Base/Math/Matrix3x3.hh>


using namespace std;
using namespace BIAS;

// entry JW
int main(int argc, char** argv){
    cout<<"started "<<argv[0]<<endl;
#ifdef BIAS_TESTS_DATA
    string filename(BIAS_TESTS_DATA "calib01.bog");
#else
    string filename("calib01.bog");
    cout<<"BIAS_TESTS_DATA  not defined."<<endl;
    return -1;
#endif
    if (argc>1) filename=argv[1];
    cout<<"using filename "<<filename<<endl;

    //bool ok=true;

    Vector3<double> C;
    if (0!=C.LoadBogTC(filename)) return -2;
    cout<<"C: "<<C<<endl;

    Matrix3x3<double> R;
    if (0!=R.LoadBogRC(filename)) return -3;
    cout<<"R: "<<R<<endl;

    Matrix3x3<double> K;
    if (0!=K.LoadBogK(filename)) return -3;
    cout<<"K: "<<K<<endl;

    // check values for correctness: 
    int result=0;
    const double v[3] = { -317.4137573 , 0.4055223 , -3.9081306 };
    const double eps=1E-10; // desired precision
    for (unsigned int i=0; i<3; i++) {
        if (fabs(v[i] - C.GetData()[i]) >eps){
           cout<<"Error: mismatch for element "<<i<<" : "<<endl
                    <<" should be: "<<v[i]
                    <<" but read : "<<C.GetData()[i]
                    <<endl;
                    result += -1*i; 
            }
        }
    cout<<"done with result="<<result<<endl;
    return result; // OK.
}
