# Try to find CMU 1394 (firewire) digital camera driver library  
#
# Available from:
# www-2.cs.cmu.edu/~iwan/1394/
#
# Once run this will define: 
#
# 1394CAMERACMU_FOUND
# 1394CAMERACMU_INCLUDE_DIR
# 1394CAMERACMU_LIBRARIES
# 1394CAMERACMU_HEADER       = This is a hack to allow command completion in MSVS
#
#  (1394CAMERACMU_LINK_DIRECTORIES: not yet...)
# 
# NOTES
# tested with version 6.2, 6.3 (on Windows XP)
#
# AUTHOR
# Jan Woetzel <http://www.mip.informatik.uni-kiel.de/~jw> (02/2004)


MESSAGE(STATUS "Find1394CameraCMU.cmake: Please note that 1394CameraCMU vars. were renamed to uppercase 1394CAMERACMU --")


IF (NOT WIN32)
  #MESSAGE(STATUS "Find1394CameraCMU.cmake: This library supports only WIN32. skipping.")
  SET(1394CAMERACMU_FOUND FALSE)
ELSE (NOT WIN32)

FIND_PATH(1394CAMERACMU_INCLUDE_DIR 1394Camera.h
  $ENV{1394CAMERACMU_DIR}/include
  $ENV{1394CAMERACMU_DIR}
  $ENV{1394CAMERACMU_HOME}/include
  $ENV{1394CAMERACMU_HOME}
  "$ENV{EXTERN_LIBS_DIR}/cmu1394/include"  
  $ENV{EXTRA}/include
  $ENV{EXTRA}
)
#MESSAGE("DBG 1394CAMERACMU_INCLUDE_DIR=${1394CAMERACMU_INCLUDE_DIR}")  
  
FIND_LIBRARY(1394CAMERACMU_LIBRARY
  NAMES 1394camera 1394Camera 1394camerad 1394Camerad
  PATHS 
  $ENV{1394CAMERACMU_DIR}/lib
  $ENV{1394CAMERACMU_DIR}
  $ENV{1394CAMERACMU_HOME}/lib
  $ENV{1394CAMERACMU_HOME}
  "$ENV{EXTERN_LIBS_DIR}/cmu1394/lib"
  $ENV{EXTRA}/lib
  $ENV{EXTRA}
)
#MESSAGE("DBG 1394CAMERACMU_LIBRARY=${1394CAMERACMU_LIBRARY}")
  
# --------------------------------

IF(1394CAMERACMU_LIBRARY)
  SET(1394CAMERACMU_LIBRARIES ${1394CAMERACMU_LIBRARY})
ELSE(1394CAMERACMU_LIBRARY)
  MESSAGE(STATUS "1394CAMERACMU library not found.")
ENDIF(1394CAMERACMU_LIBRARY)

IF(NOT 1394CAMERACMU_INCLUDE_DIR)
  MESSAGE(STATUS "1394CAMERACMU include dir not found.")
ENDIF(NOT 1394CAMERACMU_INCLUDE_DIR)

IF(1394CAMERACMU_LIBRARIES AND 1394CAMERACMU_INCLUDE_DIR)
  SET(1394CAMERACMU_FOUND TRUE)
  
  # useful to see the headers files in IDE projects
  SET( 1394CAMERACMU_HEADER 
	  "${1394CAMERACMU_INCLUDE_DIR}/1394Camera.h"
	  "${1394CAMERACMU_INCLUDE_DIR}/1394CameraControl.h"
	  "${1394CAMERACMU_INCLUDE_DIR}/1394CameraControlSize.h"
	  "${1394CAMERACMU_INCLUDE_DIR}/1394CameraControlTrigger.h"
	  "${1394CAMERACMU_INCLUDE_DIR}/1394camapi.h"
	  "${1394CAMERACMU_INCLUDE_DIR}/1394common.h"
  )
  #MESSAGE("DBG 1394CAMERACMU_HEADER=${1394CAMERACMU_HEADER}")
  SOURCE_GROUP(CMU1394 FILES ${1394CAMERACMU_HEADER})
  
ELSE(1394CAMERACMU_LIBRARIES AND 1394CAMERACMU_INCLUDE_DIR)
  SET(1394CAMERACMU_FOUND FALSE)
ENDIF(1394CAMERACMU_LIBRARIES AND 1394CAMERACMU_INCLUDE_DIR)

MARK_AS_ADVANCED(
  1394CAMERACMU_INCLUDE_DIR
  1394CAMERACMU_LIBRARY
  1394CAMERACMU_LIBRARIES
)
ENDIF (NOT WIN32)

# make FIND_PACKAGE case sensitive compatible
SET(1394CameraCMU_FOUND ${1394CAMERACMU_FOUND})
