# -Find the native TIFF includes and library
#
# This module defines
# TIFF_INCLUDE_DIR, where to find tiff.h, etc.
# TIFF_LIBRARIES, the libraries to link against to use TIFF.
# TIFF_FOUND, If false, do not try to use TIFF.
#
# Birger Streckel and Jan Woetzel

# also defined, but not for general use are
# TIFF_LIBRARY, where to find the TIFF library.


SET(TIFF_POSSIBLE_ROOT_DIRS
  $ENV{TIFF_DIR}
  "$ENV{EXTERN_LIBS_DIR}/tiff"
  "[HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\Tiff-3.8.0_is1;Inno Setup: App Path]"
  $ENV{ProgramFiles}/GnuWin32
  $ENV{ProgramFiles}/Tiff
  /usr/
  /usr/local/
  )


FIND_PATH(TIFF_INCLUDE_DIR 
  NAMES tiff.h
  PATHS ${TIFF_POSSIBLE_ROOT_DIRS}
  PATH_SUFFIXES include inc    
  )

FIND_LIBRARY(TIFF_LIBRARY 
  NAMES libtiff tiff
  PATHS ${TIFF_POSSIBLE_ROOT_DIRS}
  PATH_SUFFIXES lib lib/Debug
  DOC "tiff library"
  )


IF(TIFF_INCLUDE_DIR)
  IF(TIFF_LIBRARY)
    SET( TIFF_FOUND TRUE )
    SET( TIFF_LIBRARIES ${TIFF_LIBRARY} )
  ENDIF(TIFF_LIBRARY)
ENDIF(TIFF_INCLUDE_DIR)

# ==========================================
IF(NOT TIFF_FOUND)
  # make FIND_PACKAGE friendly
  IF(NOT TIFF_FIND_QUIETLY)
    IF(TIFF_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "TIFF required, please specify it's location.")
    ELSE(TIFF_FIND_REQUIRED)
      MESSAGE(STATUS       "ERROR: TIFF was not found.")
    ENDIF(TIFF_FIND_REQUIRED)
  ENDIF(NOT TIFF_FIND_QUIETLY)
ENDIF(NOT TIFF_FOUND)

