/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __CornerDetectorFoerstner_hh__
#define __CornerDetectorFoerstner_hh__

#include "CornerDetectorGradient.hh"

namespace BIAS {

  /** @class CornerDetectorFoerstner
      @ingroup g_feature
      @brief foerstner corner detector, detects grey value corners in images
      @author koeser 09/2004 */
  template <class StorageType, class CalculationType>
  class BIASFeatureDetector_EXPORT CornerDetectorFoerstner 
    : public CornerDetectorGradient<StorageType, CalculationType>
  {
  public:
    CornerDetectorFoerstner();
    ~CornerDetectorFoerstner();

    protected:
    /** @brief Computes the Foerstner cornerness criterion det/trace, 
        points are accepted only if corner-ellipse is also round enough
        
        c = det / trace
        Also fills up _FeatList.
        @author koeser 09/2004 
        @todo optimize for ROI usage */
      virtual int _ComputeCornerness(Image<CalculationType>& im);

 
  }; // class

} // namespace


#endif // __CornerDetectorFoerstner_hh__
