/*
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Filter/Rescale.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Debug/TimeMeasure.hh>
using namespace BIAS;
using namespace std;

#define type 1
#if type == 1
#define StorageType unsigned char
#define OutputImageType unsigned char
#else 
#define StorageType float
#define OutputImageType float
#endif

int main(int argc, char *argv[])
{
  TimeMeasure Timer1;
  Rescale<StorageType, OutputImageType> rescale;

  Image<unsigned char> ucim;
  Image<StorageType> im, dst;

  int argind = 1;
  if (argc-argind<1 || argind<1){
    cerr << argv[0] << " [parameter] <im1> <im2> [ <im3> ... ] \n";
    return -2;
  } 
  
 
  // set debuglevel
  rescale.AddDebugLevel(D_RESCALE_INIT);
  //rescale.AddDebugLevel(D_RESCALE_WRITE_IMG);

  for (int i=argind; i<argc; i++){
#if type == 1
    if (ImageIO::Load(argv[i], im)!=0){
#else
    if (ImageIO::Load(argv[i], ucim)!=0){
#endif
      BIASERR("error loading image "<<argv[i]);
      return -1;
    } else {
      cerr << "read "<<argv[i]<<endl;
    }

#if type != 1
    if (ImageConvert::ConvertST(ucim, im, ImageBase::ST_float)!=0){
      BIASERR("error converting image "<<argv[i]);
    }
#endif
    Timer1.Start();
    rescale.Filter(im, dst);
    Timer1.Stop();

    ostringstream name;
    name << "rescale-"<<setw(4)<<setfill('0')<<i-argind<<".ppm";
    ImageIO::ExportImage(name.str(), dst, ImageIO::FF_ppm);

  }

    Timer1.Print();
  return 0;
}
