/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "Morphology.hh"

#include <Base/Common/BIASpragma.hh>

using namespace BIAS;
using namespace std;

template <class InputStorageType, class OutputStorageType>
Morphology<InputStorageType, OutputStorageType>::
Morphology() 
{
  kernelSize_=3;
}

template <class InputStorageType, class OutputStorageType>
Morphology<InputStorageType, OutputStorageType>::
Morphology(const Morphology<InputStorageType, OutputStorageType>& other)
  : kernelSize_(other.kernelSize_)
{
}

template <class InputStorageType, class OutputStorageType>
Morphology<InputStorageType, OutputStorageType>::~Morphology() 
{}

template <class InputStorageType, class OutputStorageType>
int Morphology<InputStorageType, OutputStorageType>::
FillBorderConst(Image<OutputStorageType >& Image, 
                unsigned short int XBorderSize,
                unsigned short int YBorderSize,
                OutputStorageType Value) {
  register OutputStorageType *UpperAddr, *LowerAddr, *last;
  unsigned int ChannelCount= Image.GetChannelCount();
 
  last=
    &(Image.GetImageData()[(Image.GetPixelCount()-1)*ChannelCount]);

  // fill upper and lower border
  if (XBorderSize>0){
    UpperAddr=Image.GetImageData();
    LowerAddr=Image.GetImageDataArray()[Image.GetHeight()-1];;  
    while (LowerAddr<=last) {
      *UpperAddr=Value;
      *LowerAddr=Value;
      UpperAddr++; LowerAddr++;
    } 
  }

  // fill left and right border
  if (YBorderSize > 0){
    register int WidthStep = Image.GetWidth()*ChannelCount;
    UpperAddr=Image.GetImageData();
    LowerAddr=UpperAddr+WidthStep-1;
    while (LowerAddr<=last) {
        *UpperAddr=Value;
        *LowerAddr=Value;
        UpperAddr+=WidthStep; LowerAddr+=WidthStep;
	    }
  }

  return 0;
}

//////////////////////////////////////////////////////////////////////////
//                 instantiation
//////////////////////////////////////////////////////////////////////////
#define FILTER_INSTANTIATION_CLASS Morphology
#include "Filterinst.hh"
