#include <iostream>
#include <Geometry/Projection.hh>
#include <Geometry/ProjectionParametersSpherical.hh>
#include <Geometry/ProjectionParametersGreatCircles.hh>
#include <Image/ProjectionMapping.hh>
#include <Utils/Param.hh>
#include <Utils/ThreeDOut.hh>
#include <Base/Image/ImageIO.hh>
#include <Image/Camera.hh>

using namespace std;
using namespace BIAS;

//#define COLOR_INCREMENT 5

void projectionUnProjectionLocal(ProjectionParametersBase* pPB,
                                Vector3<double> localPointDir) {
  cout<<"local projection:\n";
  localPointDir.Normalize();
  cout<<"3D : "<<localPointDir<<endl;
  HomgPoint2D imgPoint = pPB->ProjectLocal(localPointDir);  
  cout<<"2D : "<<imgPoint<<endl;
  localPointDir = pPB->UnProjectLocal(imgPoint);
  cout<<"3D' : "<<localPointDir<<endl<<endl;
}

int main(int argc, char* argv[]) {

  //load camera and write out 
  Param params(true);

  int fup = params.ParseCommandLine(argc, argv);
  
  Camera<unsigned char> sphericalCamera;
  if(ImageIO::Load(argv[fup], sphericalCamera)!=0) {
    cerr<<"unable to load spherical camera\n";
    return -1;
  }
  
  sphericalCamera.ParseMetaData();
  Projection P = sphericalCamera.GetProj();
  
  
  ProjectionParametersSpherical* pPPS =
    dynamic_cast<ProjectionParametersSpherical*>(P.GetParameters());
  if(pPPS==NULL) {
    cerr<<"no spherical camera found\n";
    return -1;
  }

  cout<<"PPS: "<<endl<<*pPPS<<endl;
  unsigned int width, height;
  pPPS->GetImageSize(width, height);

  ProjectionParametersGreatCircles PPBC;
  
  Pose poseS = pPPS->GetPose();
  RMatrix RS = poseS.GetR();
  RMatrix RGC;
  cout<<"RS: "<<RS<<endl;
  for(unsigned int i=1; i<4; i++) {
    RGC(i, 1) = RS(i, 3);
    RGC(i, 2) = RS(i, 2);
    RGC(i, 3) = -RS(i, 1);
  }
  cout<<"RGC: "<<RGC<<endl;
  poseS.SetR(RGC);
  //at commented state gc-base is identity
  //uncomment this to have gc-base rotated by 90Deg around y-axis
  PPBC.SetPose(poseS);
  //uncomment this to use same pose as spherical camera
  //  PPBC.SetPose(pPPS->GetPose());
  double radius = pPPS->GetRadius();
  unsigned int size = (unsigned int)ceil(radius);
  PPBC.SetImageSize(size, size);

  HomgPoint3D zl(0,0,1,1);
  HomgPoint3D worldOpticalAxis = pPPS->GetPose().LocalToGlobal(zl);
  Vector3<double> zle;
  worldOpticalAxis.GetEuclidian(zle);

  if(PPBC.SetCenteredAngleInterval(worldOpticalAxis, 
                                   pPPS->GetMaxCamAngle() )!=0)
//                                     M_PI_2*0.8, 
//                                   pPPS->GetMaxCamAngle()*1.2)!=0)

    return -1;
//   PPBC.SetMinAngles(-M_PI_2, -M_PI_2);
//   PPBC.SetAngleRanges(M_PI, M_PI);
  

  
  cout<<"using "<<PPBC<<endl;
  ProjectionMapping<unsigned char, unsigned char> mapper;
  mapper.SetSourceCam(P);
  Projection PBC(PPBC);
  mapper.SetSinkCam(PBC);
  Camera<unsigned char> greatCircleImage;
  greatCircleImage.Init(size, size, sphericalCamera.GetChannelCount());

  mapper.Map(sphericalCamera, greatCircleImage);

  ThreeDOut tdo;
  RGBAuc color(255, 255, 255, 255);
  tdo.AddProjection(Projection(PPBC), color, 0.1, 1.0, "noname");
  HomgPoint3D imagePlanePoint;
  color.Set(0, 255, 0, 255);
  pPPS->SetC(  pPPS->GetC()+Vector3<double>(2.2, 0.0, 0.0));
  tdo.AddProjection(Projection(*pPPS), color, 0.1, 1.0,"orig");

  for(unsigned int x=0; x<width; x++) {
    for(unsigned int y=0; y<height; y++) {
      color[0] = sphericalCamera.PixelValue(x,y, 0);
      color[1] = sphericalCamera.PixelValue(x,y, 1);
      color[2] = sphericalCamera.PixelValue(x,y, 2);
      imagePlanePoint = 
        pPPS->UnProjectToImagePlane(HomgPoint2D(x,y, 1.0));
      if(Equal(imagePlanePoint.NormL2(),0.0) || imagePlanePoint[3]==0.0)
        continue;
      imagePlanePoint.Homogenize();      
      tdo.AddPoint(imagePlanePoint, color);
    }
  }


  for(unsigned int x=0; x<size; x++) {
    for(unsigned int y=0; y<size; y++) {
      color[0] = greatCircleImage.PixelValue(x,y, 0);
      color[1] = greatCircleImage.PixelValue(x,y, 1);
      color[2] = greatCircleImage.PixelValue(x,y, 2);
      if(x==0) {
        color[0] = 255;
        color[1] = 0;
        color[2] = 0;
      }
      else {
        if(y==0){
          color[0] = 0;
          color[1] = 255;
          color[2] = 0;
        }
      }
      imagePlanePoint = 
        PPBC.UnProjectToImagePlane(HomgPoint2D(x,y, 1.0));
      if(Equal(imagePlanePoint.NormL2(),0.0) || imagePlanePoint[3]==0.0)
        continue;
      imagePlanePoint.Homogenize();
      tdo.AddPoint(imagePlanePoint, color);
    }
  }

  greatCircleImage.SetProj(Projection(PPBC));
  greatCircleImage.UpdateMetaData();

  tdo.VRMLOut("bigcircle.wrl");
  if(ImageIO::ExportImage("bigcircle.ppm", greatCircleImage)!=0) {
    cerr<<"error writing image\n";
    return -1;
  }

  return 0;
}
