/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __BIAS_ProjectionParametersFactory_hh__
#define __BIAS_ProjectionParametersFactory_hh__

#include <bias_config.h>
#include <Base/Common/W32Compat.hh>

#include <Geometry/ProjectionParametersBase.hh>
#include <Geometry/ProjectionParametersIO.hh>

namespace BIAS {

  class BIASGeometry_EXPORT ProjectionParametersFactory
  {
  public:
    ProjectionParametersFactory();

    /** Creates a projection parameters object by a class name. */
    ProjectionParametersBase* Create(const std::string& className);

    /** Creates a projection parameters object by CameraData. */
    ProjectionParametersBase* Create(const CameraData& camData);

    /** Loads a projection parameters object from a file.
        Returns NULL on error. */
    ProjectionParametersBase* Load(const std::string& fileName);

    inline void GetValidClassNames(std::vector<std::string>& names) const
    { names = ValidClassNames_; }

  protected:
    std::vector<std::string> ValidClassNames_;
  };
  
} // namespace BIAS

#endif // __BIAS_ProjectionParametersFactory_hh__
