/**	Simple example for BIAS::OpenGLCanvas(Base) usage.
@author: Jan Woetzel */

#include <bias_config.h>

//#include <Base/Common/LeakChecking.h>

#include "ExampleOpenGLCanvas.hh"

// BIAS
#include <Gui/ConsoleRedirectIO.hh>
#include <Gui/GeometryGL.hh>

using namespace std;

// required for wx RTTI dynamic construction: 
IMPLEMENT_APP(ExampleOpenGLCanvasApp)

// --------------------------------------------------------
#ifndef BIAS_HAVE_GLUT
#  define GLUT_BUILDING_LIB
#  include <GL/glut.h> 
#endif // BIAS_HAVE_GLUT


///
/// your own implemnetation of the scenes GL calls: 
///

MyGLCanvas::MyGLCanvas(wxFrame* parent)
: 
BIAS::OpenGLCanvas( parent, -1,wxDefaultPosition, wxDefaultSize,0,wxT("GL"),(int*)(&attribsRGBASD) )
{}




void MyGLCanvas::DisplayCalls()
{
  //CALLINFO;
  CHECK_GL_ERROR;

  DisplayClear();
  DisplayCameraView();
  CHECK_GL_ERROR;
  glClearColor(0.3,0.1,0.1,0);
  glClear(GL_COLOR_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);

  // viewport
  //glViewport(0,0, 512,512);

  /*
  // simple vertex array 
  const float v[8][3] =
  {
    {-1,-1,-1}
    ,{ 1,-1,-1}
    ,{ 1, 1,-1}
    ,{-1, 1,-1}
    ,{-1,-1, 1}
    ,{ 1,-1, 1}
    ,{ 1, 1, 1}
    ,{-1, 1, 1}
  };
  glColor4f(0,0,1,0.5);
  glBegin(GL_QUADS);
  glColor3f(0.6, 0, 0); //r
  glVertex3fv( v[0] );
  glVertex3fv( v[1] );
  glVertex3fv( v[2] );
  glVertex3fv( v[3] );
  glColor3f(0, 0.6, 0); //g
  glVertex3fv( v[4] );
  glVertex3fv( v[5] );
  glVertex3fv( v[6] );  
  glVertex3fv( v[7] );
  glColor3f(0, 0, 0.6); //b
  glVertex3fv( v[1] );
  glVertex3fv( v[0] );
  glVertex3fv( v[4] );  
  glVertex3fv( v[5] );
  glEnd();
*/

#ifdef BIAS_HAVE_GLUT
  glColor3f(0.8,0.0,0.8);
  glutWireTeapot(0.5);  
#endif // BIAS_HAVE_GLUT

  this->DisplayCoordCross();

  //// animated scene
  //BIAS::GeometryGL::DisplayCallsSampleDisplayListScene(this->time);
  //CHECK_GL_ERROR;

  this->GetViewController().GetExternals().DisplayViewAt();

  /*
  glColor3f(0.5, 0.5, 0.5);
  //glutSolidCube(0.1);
  //glutSolidCube(1.0);

  glColor3f(0.5, 0, 0);
  glPushAttrib(GL_POINT_BIT);
  glPointSize(31);
  glBegin(GL_POINTS);
  glVertex3f(0,0,0);
  glEnd();
  glPopAttrib();

  glColor3f(0, 0.5, 0);
  glPushAttrib(GL_POINT_BIT);
  glPointSize(7);
  glBegin(GL_POINTS);
  glVertex3f(0,0,0);
  glEnd();
  glPopAttrib();
  */

}


// --------------------------------------------------------

bool ExampleOpenGLCanvasApp::OnInit()
{
  // utility: create a Redirection console for cout, cerr etc.
  ConsoleRedirectIO();
  // testing output: 
  cout<<"Hello cout"<<endl;
  cout<<"Hello cerr"<<endl;

  MyFrame *frame = new MyFrame( wxT("ExampleOpenGLCanvas"), wxPoint(0,0), wxSize(640,480) );
  frame->Show(TRUE);
  SetTopWindow(frame);

  return TRUE;
}


// --------------------------------------------------------
BEGIN_EVENT_TABLE(MyFrame, wxFrame)
EVT_MENU(ID_QUIT, MyFrame::OnQuit)
EVT_MENU(ID_ABOUT, MyFrame::OnAbout)
EVT_MENU(ID_TESTSHOT, MyFrame::OnTestShot)
END_EVENT_TABLE()
// --------------------------------------------------------


MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
: wxFrame((wxFrame *)NULL, -1, title, pos, size)
,p_canvas(NULL)
{
  // Create a simple top Menu
  wxMenu *menuFile = new wxMenu;  
  menuFile->Append( ID_ABOUT, wxT("&About...") );
  menuFile->AppendSeparator();
  menuFile->Append( ID_TESTSHOT, wxT("&Shot...\tCtrl-S") );
  menuFile->AppendSeparator();
  menuFile->Append( ID_QUIT, wxT("E&xit") );

  wxMenuBar *menuBar = new wxMenuBar;
  menuBar->Append(menuFile, wxT("&File"));
  SetMenuBar(menuBar);

  // Create a simple bottom status bar 
  CreateStatusBar(3);

  // Canvas to test: 
  //p_canvas=new BIAS::OpenGLCanvas(this, -1);
  p_canvas=new MyGLCanvas(this);

  //p_2nd_canvas = new MyGLCanvas(this);
}


void MyFrame::OnTestShot(wxCommandEvent& /*event*/)
{
  p_canvas->ScreenShot("out_RGB.jpg", true, GL_RGB);

  p_canvas->ScreenShot("out_RGB.mip", true, GL_RGB);
  p_canvas->ScreenShot("out_RGBA.mip", true, GL_RGBA);
  p_canvas->ScreenShot("out_DEPTH.mip", true, GL_DEPTH_COMPONENT);
  p_canvas->ScreenShot("out_DEPTH.jpg", true, GL_DEPTH_COMPONENT);
  p_canvas->ScreenShot("out_ALPHA.mip", true, GL_ALPHA);
  p_canvas->ScreenShot("out_ALPHA.jpg", true, GL_ALPHA);
  p_canvas->ScreenShot("out_LUMINANCE.mip", true, GL_LUMINANCE);
  p_canvas->ScreenShot("out_STENCIL.mip", true, GL_STENCIL_INDEX);
}


void MyFrame::OnQuit(wxCommandEvent& ){ 
  Close(TRUE); 
}


void MyFrame::OnAbout(wxCommandEvent& )
{
  wxString title(wxT("About"));
  wxString msg; // message
  msg<<wxT("About - ")<<AsciiToWx(FUNCNAME)<<wxT("\n");
  msg<<wxT("author: Jan Woetzel (c) 2003-2005\n");
  msg<<wxT("build: ")<<AsciiToWx(__DATE__)<<wxT(" ")<<AsciiToWx(__TIME__)<<wxT("\n");
  msg<<wxT("from ")<<AsciiToWx(__FILE__)<<wxT("\n");
  msg<<wxT("GUI using: ")<<wxVERSION_STRING<<wxT("\n");
  msg<<wxT("\n");
  COUT(msg);
  wxMessageBox(msg, title, wxICON_INFORMATION | wxOK );
}

