#include "ImageCanvasCheckerboardExtract.hh"
#include <FeatureDetector/CheckerboardDetector.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageDraw.hh>

using namespace BIAS;
using namespace std;



ICECCallbackInterface::ICECCallbackInterface()
{}

ICECCallbackInterface::~ICECCallbackInterface()
{}

void ICECCallbackInterface::CheckerboardSelected(std::vector<BIAS::HomgPoint2D> &corners,
                                                 bool bOnlyFourPointsGiven)
{
  BIASERR("error: function of derived class should be called");
}

/////////////////////////////////////////////////////////////////

ImageCanvasCheckerboardExtract::ImageCanvasCheckerboardExtract(wxWindow* parent, 
                                                               wxStatusBar* sbar,
                                                               int sbarid, wxWindowID id, 
                                                               const wxPoint& pos, 
                                                               const wxSize& size, 
                                                               long style, 
                                                               const wxString& name)
  : ImageCanvas(parent, sbar, sbarid, id, pos, size, style, name)
{
  NumLeftClicks_=0;
  SelectCheckerboard_=false;
  CallbackObj_=NULL;
  first_=0;
  second_=0;
  third_=0;
  forth_=0;
}

ImageCanvasCheckerboardExtract::~ImageCanvasCheckerboardExtract()
{}



void ImageCanvasCheckerboardExtract::
SetPatternParameters (unsigned int xCorners, unsigned int yCorners) {
  xCorners_ = xCorners;
  yCorners_ = yCorners;
}

void ImageCanvasCheckerboardExtract::
SelectCheckerboardCorners(ICECCallbackInterface& obj)
{
  BIAS_MUTEX_LOCK(origim_mutex_);
  BIASDOUT(D_IC_IM,"origim_ in SelectRectangle adr: "<<&origim_);
  BIAS_MUTEX_UNLOCK(origim_mutex_);
  
  CallbackObj_=&obj;

  //Here we try to automatically extract the checkerboard
  t_CoordVecf coords;
  int res = CheckerboardDetector::Compute(rgbim_,xCorners_,yCorners_,coords);
  if(res!= 0){
    cout<<"Could not detect all Checkerboard Corners, please set manually"<<endl;
    SelectCheckerboard_ = true;
  }
  else{
   
    first_[0] = (int)coords[0].first;
    first_[1] = (int)coords[0].second;
    second_[0]= (int)coords[xCorners_-1].first;
    second_[1]= (int)coords[yCorners_-1].second;

    third_[0] = (int)coords[(xCorners_*yCorners_-1)].first;
    third_[1] = (int)coords[(xCorners_*yCorners_-1)].second;
    
    forth_[0] = (int)coords[(yCorners_-1)*(xCorners_)].first;
    forth_[1] = (int)coords[(yCorners_-1)*(xCorners_)].second;

    checkerboardCorners_.clear();
    BIAS::HomgPoint2D point;
    for(unsigned int i=0;i< coords.size();i++){
      point[0] = (int)coords[i].first;
      point[1] = (int)coords[i].second;
      point[2] = 1;
      checkerboardCorners_.push_back(point);
    }
    CallbackObj_->CheckerboardSelected(checkerboardCorners_, false); 
    
    DrawCheckerboardCorners_(first_,second_,third_,forth_);  
    // RefreshDrawIm_();
    wxPaintEvent pe;
    wxPostEvent(this->GetEventHandler(), pe);
    NumLeftClicks_=0; 
  }
 
}


BEGIN_EVENT_TABLE (ImageCanvasCheckerboardExtract, ImageCanvas)
    EVT_LEFT_DOWN  (ImageCanvasCheckerboardExtract::OnLeftMouseButton) 
    EVT_RIGHT_DOWN  (ImageCanvasCheckerboardExtract::OnRightMouseButton)
    EVT_MOTION      (ImageCanvasCheckerboardExtract::OnMouseMove)
    EVT_PAINT       (ImageCanvasCheckerboardExtract::OnPaint)
END_EVENT_TABLE()

void ImageCanvasCheckerboardExtract::OnLeftMouseButton(wxMouseEvent &event)
{
  SelectCheckerboard_=true;
  wxClientDC dc(this);
  PrepareDC(dc);
  wxPoint pos = event.GetPosition();
  unsigned x = (unsigned) dc.DeviceToLogicalX( pos.x );
  unsigned y = (unsigned) dc.DeviceToLogicalY( pos.y );
  switch (NumLeftClicks_){
  case 0:
    first_[0]=x; first_[1]=y;first_[2] = 1;
    second_ = third_ = forth_ = 0.0;
    NumLeftClicks_++;
    break;
  case 1:
    {
      second_[0]=x; second_[1]=y; second_[2] = 1;
      NumLeftClicks_++;
      break;        
    }
  case 2:
    {
      third_[0]=x; third_[1]=y; third_[2] = 1;
      NumLeftClicks_++;
      break;        
    }
  case 3:
    {
      forth_[0]=x; forth_[1]=y; forth_[2] = 1;      
      checkerboardCorners_.clear();
      checkerboardCorners_.push_back(first_);
      checkerboardCorners_.push_back(second_);
      checkerboardCorners_.push_back(third_);
      checkerboardCorners_.push_back(forth_);

      CallbackObj_->CheckerboardSelected(checkerboardCorners_,true);       
      DrawCheckerboardCorners_(first_,second_,third_,forth_); 
      SelectCheckerboard_=false;
      NumLeftClicks_=0;      
    }
    break;
  default:
    BIASERR("something strange happend");
    break;
  }
//  RefreshDrawIm_();
//  wxPaintEvent pe;
//  wxPostEvent(this->GetEventHandler(), pe);
 
}



void ImageCanvasCheckerboardExtract::OnRightMouseButton(wxMouseEvent &event)
{
  if (SelectCheckerboard_){
    first_ = second_ = third_ = forth_ = 0.0;
    NumLeftClicks_ = 0;
    SelectCheckerboard_=false;
    RefreshDrawIm_();
    wxPaintEvent pe;
    wxPostEvent(this->GetEventHandler(), pe);
  }
  else
    event.Skip();
}

void ImageCanvasCheckerboardExtract::OnPaint(wxPaintEvent &event)
{  
  event.Skip();
}


void ImageCanvasCheckerboardExtract::OnMouseMove(wxMouseEvent &event)
{
  wxClientDC dc(this);
  PrepareDC(dc);
   
  wxPoint pos = event.GetPosition();
  int x = dc.DeviceToLogicalX( pos.x );
  int y = dc.DeviceToLogicalY( pos.y );
  if( SelectCheckerboard_){
    
    if (NumLeftClicks_==1){    
      second_[0]=x;
      second_[1]=y;   
    } 
    else if(NumLeftClicks_==2){     
      third_[0]=x;
      third_[1]=y;     
    }
    else if(NumLeftClicks_==3){     
      forth_[0]=x;
      forth_[1]=y;     
    }   
    DrawCheckerboardCorners_(first_,second_,third_,forth_);   
  }   
  
  event.Skip();
}



void ImageCanvasCheckerboardExtract::
DrawCheckerboardCorners_(BIAS::HomgPoint2D first,
                         BIAS::HomgPoint2D second, 
                         BIAS::HomgPoint2D third,
                         BIAS::HomgPoint2D forth)
{
  unsigned char red[]={255, 0, 0};
  unsigned char green[]={0,255, 0};
  unsigned char blue[]={0,0,255};

  RefreshDrawIm_();

  if(first_[0]!=0.0 && first_[1]!=0.0 && second_[0]!=0.0 && second_[1]!=0.0)
    { 
      ImageDraw<unsigned char>::Line(rgbim_, (int)first_[0], (int)first_[1], 
                                     (int)second_[0],(int)second_[1], green,5);

      if(third_[0] != 0.0 && third_[1] != 0.0)  
        ImageDraw<unsigned char>::Line(rgbim_, (int)second_[0], (int)second_[1], 
                                       (int)third_[0],(int)third_[1], green,5);
      if(forth_[0] != 0.0 && forth_[1] != 0.0){
        ImageDraw<unsigned char>::Line(rgbim_, (int)third_[0], (int)third_[1], 
                                       (int)forth_[0],(int)forth_[1], blue,5);
        ImageDraw<unsigned char>::Line(rgbim_, (int)forth_[0],(int)forth_[1],
                                       (int)first_[0], (int)first_[1],red,5);
      }
  
    }
  wxPaintEvent pe;
  wxPostEvent(this->GetEventHandler(), pe);
}


void ImageCanvasCheckerboardExtract::RefreshDrawIm_()
{
  Image<unsigned char> tmprgb;
  if (origim_.GetColorModel()!=ImageBase::CM_RGB){
    if (ImageConvert::Convert(origim_, tmprgb, ImageBase::CM_RGB)!=0){
      BIASERR("error converting");
    }
  } else {
    tmprgb=origim_;
  }
  Show(origim_, tmprgb, ImageName_);
}
