#ifndef __ParamGUI_hh__
#define __ParamGUI_hh__

#include <bias_config.h>
#include <Base/Common/BIASpragmaStart.hh>

// STL headers
#include <map>

// BIAS headers
#include <Utils/ParamTypes.hh>
#include <Utils/Param.hh>

// WX headers
#include <wx/wx.h>
#include <wx/spinctrl.h>
#include <wx/notebook.h>

namespace BIAS
{
  struct BIASGui_EXPORT ItemAndWidget {
    wxControl           *widget;
    BIAS::ParamTypeBase *item;
  };

  /** @class ParamGUIPanel
  @ingroup g_gui
  @brief ??
  @author evers/mdunda */
  class BIASGui_EXPORT ParamGUIPanel : public wxPanel
  {
  public:
    ParamGUIPanel(wxWindow *parent);
    ~ParamGUIPanel();

    // methods for adding widgets
    void AddIntWidgets(BIAS::ParamTypeInt *item);
    void AddBoolWidgets(BIAS::ParamTypeBool *item);
    void AddDoubleWidgets(BIAS::ParamTypeDouble *item);
    void AddStringWidgets(BIAS::ParamTypeString *item);
    void AddVecDblWidgets(BIAS::ParamTypeVecDbl *item);
    void AddVecIntWidgets(BIAS::ParamTypeVecInt *item);
    void AddEnumWidgets(BIAS::ParamTypeEnum *item);

  protected:
    wxWindow *Parent_;
  private: // avoid DLL export
    std::map<unsigned int, struct ItemAndWidget> WidgetIDs_;
  protected:
    wxFlexGridSizer *Grid_;

    // event handling methods
    void OnSpinCtrl(wxSpinEvent &event);
    void OnCheckBox(wxCommandEvent &event);
    void OnTextEnter(wxCommandEvent &event);
    void OnChoice(wxCommandEvent &event);

    // WX macro to declare an event table for this object
    DECLARE_EVENT_TABLE()
  };

  class BIASGui_EXPORT ParamGUI;

  class BIASGui_EXPORT ParamGUIDialog : public wxDialog
  {
  public:
    ParamGUIDialog(wxWindow *parent,const wxString& title, 
      const wxPoint &p,const wxSize &s, ParamGUI *pg);
    ~ParamGUIDialog();
  protected:

    DECLARE_EVENT_TABLE()
  };

  /** @class ParamGUI
  @ingroup g_gui
  @brief ??
  @author evers/mdunda */
  class BIASGui_EXPORT ParamGUI : public BIAS::Param
  {
  public:
    ParamGUI();
    ~ParamGUI();

    /** Builds up a wxPanel with the parameters specified by GroupID.
    @return a pointer to the new panel 
    @return NULL on error
    @author mdunda 03 2004 */
    wxPanel* GetPanel(wxWindow *parent, int GroupID=GRP_ALL);

    /** Builds up a wxNotebook with each parameter group on one page.
    Group-ID -1 (GRP_NOSHOW) will be ignored.
    @return a pointer to the new notebook
    @retrun NULL on error
    @author mdunda 03 2004 
    now GroupNames are shown in tabs if all Groupnames are set
    ischiller 08.09.04
    */
    wxNotebook* GetNotebook(wxWindow *parent);

    /** Builds up a standalone wxFrame with a notebook embedded in it.
    @return a pointer to the new frame
    @retrun NULL on error
    @author mdunda 03 2004 */
    wxFrame* GetFrame(wxWindow *parent);


    /** Builds up a  wxDialog with a notebook embedded in it.
    @return a pointer to the new dialog
    @retrun NULL on error
    @author evers  2004-01 */
    ParamGUIDialog* GetDialog(wxWindow *parent, const wxString &title);


  }; // class
} // namespace

#include <Base/Common/BIASpragmaEnd.hh>

#endif /* __ParamGUI_hh__ */
