#ifndef __VideoSource_Controller_HH__
#define __VideoSource_Controller_HH__

#include <bias_config.h>

#include <Base/Common/W32Compat.hh>
#include <VideoSource/VideoSource_Base.hh>

#ifdef BIAS_HAVE_PMDUSB
#include <VideoSource/VideoSource_usbPMD.hh>
#endif

#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageConvert.hh>

#include <Image/Camera.hh>
#include <Image/StereoRedGreen.hh>

#ifndef BIAS_HAVE_WXWIDGETS
#  error You need BIAS_HAVE_WXWIDGETS. Recompile BIAS with USE_WXWIDGETS.
#endif

#include <wx/wx.h>



namespace BIAS {

class BIASGui_EXPORT VideoSource_Controller : public ::wxFrame{
public:

  VideoSource_Controller(BIAS::VideoSource *cam, wxFrame *parent, 
		double minShutter=1.0, double maxShutter=80.0,
		double minGain = 0.0,double maxGain=3000.0);
  ~VideoSource_Controller();

  void Initialize();

  void SetVideoSource(BIAS::VideoSource *cam);

  void Update(); // trigger an update, useful for auto-mode
  
  void SetShutterRange(double minShutter, double maxShutter);

  wxBoxSizer        *vBox;
    
  enum {
    ID_HIDE,
    //All 16 FEATURES of DCAM
    //GAIN, SHUTTER PANEL
    ID_GAIN,
    ID_SHUTTER,
    ID_BUTTON_AUTO_GAIN,
    ID_BUTTON_AUTO_SHUTTER,
    ID_BUTTON_UPDATE,
  
    //HUE, SATURATION
    ID_HUE,
    ID_SATURATION,

    //GAMMA, BRIGHTNESS
    ID_GAMMA,
    ID_BRIGHTNESS,

    ID_EXPOSURE,
    ID_SHARPNESS,
    ID_WHITE_BALANCE,
    ID_IRIS,
    ID_FOCUS,
    ID_TEMPERATURE,
    ID_TRIGGER,
    ID_TRIGGER_DELAY,
    ID_WHITE_SHADING,
    ID_FRAME_RATE,

    ID_TEXT
  };

private:

  void OnEscape(wxCommandEvent& event);
  
  void OnHide(wxCommandEvent& event);
  void OnClose(wxCloseEvent& event);
  
  void OnShutter(wxScrollEvent& event);
  void OnGain(wxScrollEvent& event);
  void OnAuto_Shutter(wxCommandEvent& event);
  void OnAuto_Gain(wxCommandEvent& event);
  void OnUpdate(wxCommandEvent& event) ;

  void auto_on(int param);
  void auto_off(int param);

protected:
  
  BIAS::VideoSource *Cam_;
  
  wxSlider          *SliderGain;
  wxSlider          *SliderShutter;
  wxBoxSizer        *sliderBox_ShGa;
  bool               sliderBox_ShGa_Visible;
  bool               auto_shutter;
  bool               auto_gain;
  
  bool               vBoxVisible;

  wxButton          *showButton;
  wxButton          *auto_sh_Button;
  wxButton          *auto_ga_Button;
  wxButton          *update_Button;

  double minShutter_, maxShutter_, minGain_,maxGain_;

  DECLARE_EVENT_TABLE()

private:

    };

}
#endif
