/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __BIAS_CylinderMapping_hh__
#define __BIAS_CylinderMappibg_hh__

// BIAS includes
#include <Base/Image/Image.hh>
#include <Image/BackwardMapping.hh> 
#include <Geometry/PMatrix.hh>

namespace BIAS {

  /** @class CylinderMapping 
      @brief functions for spherical undistortion of given images, 
      needs image dimensions,
      principal point and focal length as input additional 
      to the image data. 

      Useful for PanoramaMapping !
      See also BIAS::BackwardMapping for further details.
    
      @author jbr, february 2006 edited by grest
  */
  class BIASImage_EXPORT CylinderMapping : 
    public BIAS::BackwardMapping<unsigned char, unsigned char> { 
    
  public:

    CylinderMapping();

    /** Set the focal length of the camera.
        Defaults to 140 degrees field of view in y-direction
        @param fx
        @param fy
    */
    void setFocalLength(const double fx, const double fy);
  

    /** Set the principal point of the camera
        @param px defaults to half image size
        @param py defaults to half image size
    */
    void setPrincipalPoint(const double px, const double py);


    /**    if you want to call Map() twice for this object with different
           params, call Reset() before
    */
    void Reset();
    
  protected:

    /** this does the trick
     */
    virtual int GetSourceCoordinates_(const BIAS::HomgPoint2D& sink, 
                                      BIAS::HomgPoint2D& source) const;

    double px_;          // principal x
    double py_;          // principal y

    double focalX_;
    double focalY_;
    double camAngleX_;   // camera opening angle in x-direction
    double camAngleY_;   // camera opening angle in y-direction

};

}
#endif
