//BIAS includes 
#include <Image/ContourBSpline.hh>


#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>
#include <Base/Image/ImageBase.hh>
#include <Base/Image/Image.hh>


#include <Gui/GuiEnhanced.hh>


#include <string>

//gui
BIAS::GuiEnhanced win_;

//images
BIAS::Image<unsigned char> imgOrig_;
BIAS::Image<unsigned char> imgDraw_;
BIAS::Image<unsigned char> imgGrey_;

//B-Spline curve
BIAS::ContourBSpline bspline_;

//color
unsigned char blue_[3]={0,0,255};
unsigned char red_[3]={255,0,0};

void fit(){
  bspline_.SetShapeSpaceIdentity();
  bspline_.SetSubShapeSpacePlanarAffin();
  bspline_.ComputeRegularisationMatrix();
  bspline_.SetFitSampleWidth(0.01);
  //fit with alpha=0.001 normalWidth=50
  bspline_.Fit(imgGrey_,0.001,50);
}

void loadInputImage(char *imgloc){
  win_.SetTitle(std::string(imgloc));
  BIAS::ImageIO::Load(imgloc,imgOrig_);
  BIAS::ImageIO::Load(imgloc,imgDraw_);
  BIAS::ImageConvert::Convert(imgOrig_,imgGrey_,BIAS::ImageBase::CM_Grey);
  win_.ShowImage(imgDraw_,0,255);
}

void checkEvents(){
  bool checkEvent=true;
  //event stuff
  bool keyEvent;
  char key;
  bool pressed;
  int  button;
  int  xPos;
  int  yPos;

  while(checkEvent){
    win_.WaitForEvent(keyEvent, key, pressed, button, xPos, yPos);
    if(keyEvent){
      switch(key){
      case 27: //ESC - quit program
        win_.DestroyWindow();
        exit(EXIT_SUCCESS);
        break;
      case 'f': //fit spline
        imgDraw_=imgOrig_;
        fit();
        bspline_.DrawCurve(imgDraw_,red_);
        win_.ShowImage(imgDraw_);
      }
    }
  }
}


int main(int argc, char *argv[])
{
  if(argc==1){
    std::cout<< "usage: " << argv[0] << " example.ppm" << std::endl <<
      "  press f for fitting" << std::endl << std::flush;
    exit(EXIT_FAILURE);
  }
  
  //load example.mip
  loadInputImage(argv[1]);
  
  /** construct curve */
  //build multiplicity vector
  std::vector<unsigned int> mPnts(5,1);
  //build control point vectors
  BIAS::Vector<double>Q(10);
  //x-values
  Q[0]=215;
  Q[1]=162;
  Q[2]=167;
  Q[3]=264;
  Q[4]=287;
  //y-values
  Q[5]=138;
  Q[6]=163;
  Q[7]=243;
  Q[8]=247;
  Q[9]=150;

  bspline_.Init(3, BIAS::ContourBSplineType::Closed, mPnts, Q);
  bspline_.SetDrawSampleWidth(0.01);
  bspline_.DrawCurve(imgDraw_, blue_);
  win_.ShowImage(imgDraw_);

  checkEvents();

  win_.DestroyWindow();
  return 0;
}
