#include <string>
#include <iostream>

// bias includes
#include <Utils/Param.hh>
#include <Base/Image/ImageIO.hh>
#include <Image/CylinderMapping.hh>

using namespace BIAS;
using namespace std;

BIAS::Param para; 


void ParamInit(int &args, char **&arg) // $Init
{
  // no warning at program exit
  para.DisableDestructorWarning();

  // add params (as unclassified param group)
  para.AddParamBool("help","prints info about all params",false,'h');  
  para.AddParamString("image","mip file to load","",'i');
  para.AddParamInt("threshold","some threshold");
  para.AddParamDouble("focal", "focal length", 100.0,-1,1000000,'f');
  para.AddParamDouble("hx", "principal point x", -1,-1,1E06);
  para.AddParamDouble("hy", "principal point y", -1,-1,1E06);
  
  // get command line params
  para.ParseCommandLine(args,arg);
  
  // ? print help informations
  if ( *para.GetParamBool("help") == true ) {
    cout << endl;
    cout << "Press 'h' for a list of possible keys." << endl;    
    para.Usage();
    exit(0);
  }
}

int main(int args, char **arg)
{

  // init bias param class
  ParamInit(args,arg); // modifies params

  // show image, iff give
  if ( para.GetParamString("image")->size() ==  0 ) {
    cout << "no input image specified (try --help)" << endl;
    return 0;
  }

  // read and show image
  BIAS::Image<unsigned char> image;
  int r = BIAS::ImageIO::Load(*para.GetParamString("image"),image);
  if ( r != 0 ) {
    BIASERR("error loading image: "<<(*para.GetParamString("image")));
  }

  BIAS::Image<unsigned char> sdest(image.GetWidth(), 
                                   image.GetHeight(), image.GetChannelCount());
  sdest.Clear(0);
  
  cout << "image width = " << image.GetWidth() << endl;
  cout << "image height = " << image.GetHeight() << endl; 
  
  // DEBUG
  // set the main eye points of the camera
  double hx = *(para.GetParamDouble("hx"));
  cout << "Setting hx to " << hx << endl; 

  double hy = *(para.GetParamDouble("hy"));
  cout << "Setting hy to " << hy << endl; 


  // perform the mapping
  //
  // 
  //
  CylinderMapping smapping;
  smapping.setPrincipalPoint(hx, hy);
  smapping.setFocalLength(*para.GetParamDouble("focal"), 
                          *para.GetParamDouble("focal"));
  smapping.Map(image, sdest);

  BIAS::ImageIO::ExportImage(std::string("cyl_") + 
                             *para.GetParamString("image"),
                             sdest,
                             BIAS::ImageIO::FF_ppm);
  cout<<"Mapped image written to:"<<"cyl_" <<
    *para.GetParamString("image")<<endl;
}

