/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Base/Common/BIASpragmaStart.hh>

#ifndef __LogPolarMapping_hh__
#define __LogPolarMapping_hh__

#include <Image/BackwardMapping.hh>


namespace BIAS {

  /** @class LogPolarMapping
      @ingroup g_filter
      @brief  Maps cartesian source coordinates to log-polar sink coordinates.
       The center of rotation must be set. The minimum center value is the
       radius of the circle. 
      @author oniemann
  */ 
  template <class InputStorageType, class OutputStorageType> 
  class BIASImage_EXPORT LogPolarMapping 
    : public  BackwardMapping<InputStorageType, OutputStorageType>  {
  public:

    virtual ~LogPolarMapping();

    LogPolarMapping();

    /**  required because assignment is 
         @author Jan Woetzel */
    LogPolarMapping(const LogPolarMapping & src);

    /**  required because of const members 
         @author Jan Woetzel */
    LogPolarMapping<InputStorageType, OutputStorageType> & 
    operator=(const LogPolarMapping & src);

    /** @brief set your image center before calling Map(). min(x,y) = radius
         of circle.*/
    void SetImageCenter(const int & x, const int & y);

  protected:
    /** @brief reimplementation for Pixel transformation, 
        takes sink and computes coords in source */
    virtual int GetSourceCoordinates_(const HomgPoint2D& sink,
                                      HomgPoint2D& source) const;
    int XCenter_,YCenter_;
    double R_, A_;
    HomgPoint2D *_XYMap;

  };
} // namespace 

#endif

#include <Base/Common/BIASpragmaEnd.hh>

