/** 
   This file is part of the BIAS library (Basic ImageAlgorithmS).

   Copyright (C) 2003, 2004    (see file CONTACTS for details)
   Multimediale Systeme der Informationsverarbeitung
   Institut fuer Informatik
   Christian-Albrechts-Universitaet Kiel


   BIAS is free software; you can redistribute it and/or modify
   it under the terms of the GNU Lesser General Public License as published by
   the Free Software Foundation; either version 2.1 of the License, or
   (at your option) any later version.
   
   BIAS is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public License
   along with BIAS; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <Image/RectificationBase.hh>
#include <Image/PlanarRectification.hh>
#include <Geometry/ProjectionParametersPerspective.hh>

using namespace BIAS;
using namespace std;

#define SUPERPARENT RectificationBase<InputStorageType, OutputStorageType>
#define PARENT RectificationViaProjectionMappingBase<InputStorageType, OutputStorageType>


template <class InputStorageType, class OutputStorageType>
BIAS::PlanarRectification<InputStorageType, OutputStorageType>::
PlanarRectification()
{ }

template <class InputStorageType, class OutputStorageType>
BIAS::PlanarRectification<InputStorageType, OutputStorageType>::
~PlanarRectification() 
{ }

template <class InputStorageType, class OutputStorageType>
bool BIAS::PlanarRectification<InputStorageType, OutputStorageType>::
IsInputCameraValid(const BIAS::Image<InputStorageType>& img,
                   const BIAS::ProjectionParametersBase* proj)
{
  const ProjectionParametersPerspective* ppp = 
    dynamic_cast<const ProjectionParametersPerspective*>(proj);
 
  if (ppp == NULL) {
    return false;
  }

  return true;
}


template <class InputStorageType, class OutputStorageType>
int BIAS::PlanarRectification<InputStorageType, OutputStorageType>::
DetermineRectificationParameters_()
{

  Pose rectPoseA, rectPoseB;//, relativeTransformA, relativeTransformB;
  BIASASSERT((SUPERPARENT::ppBA_)!=NULL);
  BIASASSERT((SUPERPARENT::ppBB_)!=NULL);
  if (SUPERPARENT::CalculateRectifiedBases(SUPERPARENT::ppBA_->GetPose(), 
                                           SUPERPARENT::ppBB_->GetPose(),
                                           rectPoseA, rectPoseB, true) != 0) {
    BIASERR("Failed calculating rectified base");
    return -1;
  }

  delete PARENT::rectPPA_;
  delete PARENT::rectPPB_;
  PARENT::rectPPA_ = new ProjectionParametersPerspective(); 
  PARENT::rectPPB_ = new ProjectionParametersPerspective();

  const ProjectionParametersPerspective* ppPA_ =
    dynamic_cast<const ProjectionParametersPerspective*> (SUPERPARENT::ppBA_);
   BIASASSERT(ppPA_ != NULL);
  const ProjectionParametersPerspective* ppPB_ =
    dynamic_cast<const ProjectionParametersPerspective*> (SUPERPARENT::ppBB_);
   BIASASSERT(ppPB_ != NULL);

   //the whole thing should be optimized/corrected! :
  KMatrix Ka = ppPA_->GetK();

  ProjectionParametersPerspective* rectPPPA_ =
    dynamic_cast<ProjectionParametersPerspective*> (PARENT::rectPPA_);
  ProjectionParametersPerspective* rectPPPB_ =
    dynamic_cast<ProjectionParametersPerspective*> (PARENT::rectPPB_);

  //assume that images have very low roll component relative to each other  
  // and that baseline direction is basically in direction of image width:
  unsigned int widthA, heightA;
  ppPA_->GetImageSize(widthA, heightA);
  unsigned int widthB, heightB;
  ppPB_->GetImageSize(widthB, heightB);
  int numSamplesOnEpiLine = (widthA>widthB) ? widthA : widthB;
  int numEpiLines = (heightA>heightB) ? heightA : heightB;;
  
  PARENT::rectPPA_->SetImageSize((unsigned int)numSamplesOnEpiLine, 
                                 (unsigned int)numEpiLines);
  PARENT::rectPPB_->SetImageSize((unsigned int)numSamplesOnEpiLine, 
                                 (unsigned int)numEpiLines);


  rectPPPA_->SetPose(rectPoseA);
  rectPPPA_->SetK(Ka);

  rectPPPB_->SetPose(rectPoseB);
  rectPPPB_->SetK(Ka);

  return 0;
}


template class BIASImage_EXPORT PlanarRectification<unsigned char, unsigned char>;
template class BIASImage_EXPORT PlanarRectification<float, float>;
template class BIASImage_EXPORT PlanarRectification<unsigned char, float>;
template class BIASImage_EXPORT PlanarRectification<float, unsigned char>;

#if defined(BUILD_IMAGE_CHAR)
template class BIASImage_EXPORT PlanarRectification<unsigned char, char>;
template class BIASImage_EXPORT PlanarRectification<char, char>;
#endif

#if defined(BUILD_IMAGE_USHORT)
template class BIASImage_EXPORT PlanarRectification<unsigned short,unsigned short>;
#endif

#if defined(BUILD_IMAGE_SHORT)
template class BIASImage_EXPORT PlanarRectification<short, short>;
#endif

#if defined(BUILD_IMAGE_SHORT)&&defined(BUILD_IMAGE_USHORT)
template class BIASImage_EXPORT PlanarRectification<unsigned short, short>;
#endif

#if defined(BUILD_IMAGE_INT)
template class BIASImage_EXPORT PlanarRectification<int,int>;
#endif

#if defined(BUILD_IMAGE_USHORT)
template class BIASImage_EXPORT PlanarRectification<unsigned short,float>;
#endif

#if defined(BUILD_IMAGE_USHORT) && defined(BUILD_IMAGE_INT)
template class BIASImage_EXPORT PlanarRectification<unsigned short,int>;
#endif

#if defined(BUILD_IMAGE_DOUBLE)
template class BIASImage_EXPORT PlanarRectification<double,double>;
#endif


