/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
Multimediale Systeme der Informationsverarbeitung
Institut fuer Informatik
Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#ifndef __PlanarRectification_hh__
#define __PlanarRectification_hh__

#include <Image/RectificationViaProjectionMappingBase.hh>

namespace BIAS {

  /** 
   * Rectification via perspective cameras, removes distortions.
   * Only works with perspective cameras
   * \ingroup Stereo
   * \author bartczak 03/2007
   */
  template <class InputStorageType, class OutputStorageType> 
  class BIASImage_EXPORT PlanarRectification : 
  public RectificationViaProjectionMappingBase<InputStorageType, 
                                                 OutputStorageType>
  {
  public:
    PlanarRectification();
    ~PlanarRectification();
      
    /** Only works with perspective cameras */
    virtual bool 
    IsInputCameraValid(const BIAS::Image<InputStorageType>& img,
                       const BIAS::ProjectionParametersBase* proj);
    /** @} */
  protected:

    /** current implementation is suboptimal, a beter way for the choise of
    * internal parameters must be found. Currently the CameraA parameters without
    * the distortion are chosen. 
    */
    virtual int DetermineRectificationParameters_();  
    
  };


}//end of namespace

#endif
