#include <bias_config.h>

#include <Gui/ImageCanvas.hh>
#include <Gui/StringConv.hh>
#include <Image/Camera.hh>
#include <Image/StereoRedGreen.hh>
#include <Base/Common/W32Compat.hh>
#include <VideoSource/VideoSource_Net.hh>
#include <VideoSource/VideoSource_Shm.hh>
#include <Base/Image/ImageConvert.hh>
#include <Gui/VideoSource_Controller.hh>
#include <Gui/ConsoleRedirectIO.hh>


#ifdef WIN32
#include <Base/Common/getopt_W32.h>  
//#define HAVE_TRIGGER_CLASSES // uncomment this if trigger classes are 
    //in BIAS/Tools, uncomment also linkage in CMakeLists.txt
#ifdef HAVE_TRIGGER_CLASSES
#include "TriggerIOControl.h"
#endif
#endif

// cideo for linux
#ifdef BIAS_HAVE_V4L
#  include <VideoSource/VideoSource_V4L.hh>
#endif // BIAS_HAVE_V4L

//include direct show and ueye interface?
#ifdef BIAS_HAVE_DSHOW
#  include <VideoSource/VideoSource_DSHOW.hh>
#endif

#  ifdef BIAS_HAVE_UEYE
#    include <VideoSource/VideoSource_uEye.hh>
#  endif // BIAS_HAVE_UEYE

#ifdef BIAS_HAVE_DCAM
#  include <VideoSource/VideoSource_DCAM.hh>
#endif

#ifdef BIAS_HAVE_DV
#  include <VideoSource/VideoSource_DV.hh>
#endif

#ifdef BIAS_HAVE_PMD
#  include <Image/PMDImageProc.hh>
#  include <VideoSource/VideoSource_DcamPMD.hh>
#  include <VideoSource/VideoSource_ShmPMD.hh>
#endif

#ifdef BIAS_HAVE_PMDUSB
#  include <VideoSource/VideoSource_usbPMD.hh>
#endif

#include <iostream>
#include <Base/Image/ImageIO.hh>
#include <Base/Debug/TimeMeasure.hh>
#include <wx/wx.h>
#include <wx/listbox.h>
#include <wx/thread.h>

#ifdef WIN32
#  include <Base/Common/getopt_W32.h>
#endif

using namespace std;
using namespace BIAS;


class ImageSaveThread : public wxThread {
  public:
    ImageSaveThread(wxString filename, Camera<unsigned char> img): wxThread(wxTHREAD_DETACHED)  
    {
      img_ = img;
      filename_ = filename;
    }
    ~ImageSaveThread() {}
  	void OnExit() {
	  }

    /** This method is required for each wxThread derived object.
     */
    virtual ExitCode Entry() {
      ImageIO::Write(WxToAscii(filename_),img_);
      return NULL;
    }
private:
  wxString filename_;
  Camera<unsigned char> img_;

};




enum wxIDs {id_quit = 42,id_record,id_config, id_timer,id_save,id_key,id_save2,
	    id_pin,id_textgain,id_textshutter,id_gain,id_shutter, id_show1, 
	    id_show2, id_show3, id_show4, id_controller, id_controller2, id_controllerPMDUSB};  


ofstream logstr;

int delay_ =5;
bool bSaveImages_;
bool bShowStereo_ = false;



class MyFrame : public wxFrame{
public:
  MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size, 
    VideoSource *cam, VideoSource *cam2, VideoSource *camPMD, VideoSource *camPMDShm,
    VideoSource *camPMDUSB, bool trig)
    :wxFrame((wxFrame *)NULL, -1, title, pos, size)
  {
    Timer_.SetOwner(this, id_timer);
    FrameCounter_ = 0;
    GrabCounter_ = 0;

    redgreen_=false;
    trig_ = trig;
    Cam_=NULL;
    Cam2_ = NULL;
#ifdef BIAS_HAVE_PMD
    CamPMD_ = NULL;
    CamPMDShm_=NULL;
#endif
#ifdef BIAS_HAVE_PMDUSB
    CamPMDUSB_ = NULL;
#endif
    CamSel_ = NULL;
    if(cam != NULL) { CamSel_ = cam; NumCamSel_ = 1;}
    else if(cam2 != NULL) {CamSel_ = cam2; NumCamSel_ = 2;}
#ifdef BIAS_HAVE_PMD
    else if(camPMD != NULL) {CamSel_ = camPMD; NumCamSel_ = 3;}
    else if(camPMDShm != NULL) {CamSel_ = camPMDShm; NumCamSel_ = 3;}
#endif
#ifdef BIAS_HAVE_PMDUSB
    else if(camPMDUSB != NULL) {CamSel_ = camPMDUSB; NumCamSel_ = 4;}
#endif
    
    VideoController_ = NULL;
    VideoController2_ = NULL;
    VideoControllerPMDUSB_ = NULL;
#ifdef BIAS_HAVE_DSHOW
    if( (dynamic_cast<VideoSource_DSHOW*>(Cam_)) != NULL)
    {
      (dynamic_cast<VideoSource_DSHOW*>(Cam_))->InitImage(CamImg_);
    }
    else
#endif
    if(cam != NULL){ 
        Cam_ = cam;
        Cam_->InitImage(CamImg_);
    }
    else
      Cam_ = NULL;
    if (cam2 != NULL){
      Cam2_ = cam2;
      Cam2_->InitImage(CamImg2_);
    } else {
      Cam2_ = NULL;
    }
#ifdef BIAS_HAVE_PMD
    if (camPMD!= NULL){
      CamPMD_ = camPMD;
      CamPMD_->InitImage(CamImgPMD_);
    } else {
      CamPMD_ = NULL;
    }
    if(camPMDShm != NULL){
      CamPMDShm_ = camPMDShm;
      CamPMDShm_->InitImage(CamImgPMD_);
    } else {
      CamPMDShm_ = NULL;
    }
#endif
#ifdef BIAS_HAVE_PMDUSB
    if (camPMDUSB!= NULL){
      CamPMDUSB_ = camPMDUSB;
      CamPMDUSB_->InitImage(CamImgPMDUSB_);
    } else {
      CamPMDUSB_ = NULL;
    }
#endif

    wxMenu *menuFile = new wxMenu;

    menuFile->Append( id_save, wxT("Grab single image") ); 
    menuFile->Append( id_save2, wxT("Grab continuously") );

    menuFile->Append( id_quit, wxT("E&xit") );

    wxMenu *menuShow = new wxMenu;
     
    if (Cam_ != NULL) {
      menuShow->Append( id_show1, wxT("Camera 1") ); 
      if (cam->IsControllable()){ 
        VideoController_ = new VideoSource_Controller(Cam_,this);
        VideoController_->Show(false);
        menuShow->Append(id_controller,wxT("Camera Controls")); 
      } 
    }
    if (Cam2_ != NULL){
      menuShow->Append( id_show2, wxT("Camera 2") );
      if (cam2->IsControllable()){ 
        VideoController2_ = new VideoSource_Controller(Cam2_, this);
        VideoController2_->Show(false);
        menuShow->Append(id_controller2, wxT("Camera 2 Controls")); 
      } 
    }
#ifdef BIAS_HAVE_PMD
    if (camPMD != NULL){
      menuShow->Append( id_show3, wxT("Camera PMD") );
    }
#endif
#ifdef BIAS_HAVE_PMDUSB
    if (camPMDUSB != NULL){
      menuShow->Append( id_show4, wxT("Camera PMDUSB") );
      VideoControllerPMDUSB_ = new VideoSource_Controller(CamPMDUSB_,this,1.0,80.0,1.0,80.0);
      VideoControllerPMDUSB_->Show(false);	   
      menuShow->Append(id_controllerPMDUSB,wxT("PMDUSB-Camera Controls")); 
    }
#endif

    wxMenuBar *menuBar = new wxMenuBar;
    menuBar->Append(menuFile, wxT("&File"));
    menuBar->Append(menuShow, wxT("&Show"));
#ifdef BIAS_HAVE_DSHOW
    if( (dynamic_cast<VideoSource_DSHOW*>(Cam_)) != NULL)
    {
      wxMenu * menuDShow  = new wxMenu;
      menuDShow->Append( id_config, wxT("Display DSHOW camera configuration"));
      menuDShow->Append( id_pin, wxT("Display DSHOW source pin configuration"));
      menuBar->Append(menuDShow, wxT("&DShow"));
    } 
#endif
    SetMenuBar(menuBar);
    CreateStatusBar(5);
    //DBG?: what if Cam does not exist?
    lastwidth = CamImg_.GetWidth();
    lastheight = CamImg_.GetHeight();
 
    IC_ = new ImageCanvas(this,GetStatusBar(), 0,-1,wxDefaultPosition,
                          //wxSize(lastwidth,lastheight));
                          wxSize(640,480));
    //lastwidth = CamImgPMD_.GetWidth();
    //lastheight = CamImgPMD_.GetHeight();
    //IC_ = new ImageCanvas(this,GetStatusBar(), 0,-1,wxDefaultPosition,
    //                      //wxSize(lastwidth,lastheight));
    //                      wxSize(640,480));

    //    MyControls_ = new VideoSource_Controller(Cam);


    wxBoxSizer *vbox = new wxBoxSizer(wxVERTICAL);

    vbox->Add(IC_,1,  wxEXPAND |wxALL |wxFIXED_MINSIZE , 5);
    //vbox->SetSizeHints(this);
    vbox->SetMinSize(640,480);
    SetSizer(vbox);
    SetAutoLayout(true);
    Layout();  
    Show(TRUE);

    wxString text;
    text.Printf(wxT("size: %4dx%4d"),CamImg_.GetWidth(), CamImg_.GetHeight());
    SetStatusText(text, 2);


    bSaveImages_ = false;
    cout <<"Setting timer to "<<delay_<<" ms"<<endl;
    Timer_.Start(delay_,true);
    StopWatch_.Start();
  }




  void OnKey(wxKeyEvent& event)
  {
    int code = event.GetKeyCode();
    switch(code){
case '1':
#ifdef BIAS_HAVE_DSHOW
      if(Cam_!=NULL)
        {
          if((dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_)) != NULL)
            (dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_))->DisplayCameraProperties();
        }
#endif
      break;
#ifdef WIN32
    case 83:
#else //WIN32
    case 's':
#endif //WIN32
      FrameCounter_ = 0;
      GrabCounter_ = 0;
      bSaveImages_ = !bSaveImages_;
      cout<<"saving images...= "<<bSaveImages_<<endl;
      break;
    }  
  } 

  void OnQuit(wxCommandEvent& WXUNUSED(event))
  {
    Timer_.Stop();
    if(Cam_){
      Cam_->PostGrab();     
      Cam_->CloseDevice();
      //delete Cam_;
    }
    if (Cam2_){
      Cam2_->PostGrab();     
      Cam2_->CloseDevice();
    }
#ifdef BIAS_HAVE_PMD
    if (CamPMD_){
      CamPMD_->PostGrab();     
      CamPMD_->CloseDevice();
    }
    if(CamPMDShm_)
    {
      CamPMDShm_->PostGrab();     
      CamPMDShm_->CloseDevice();
    }
#endif
#ifdef BIAS_HAVE_PMDUSB
    if (CamPMDUSB_){
      CamPMDUSB_->PostGrab();     
      CamPMDUSB_->CloseDevice();
    }
#endif
    //if(IC_)
    //delete IC_;
    Close(TRUE);
  }

  void OnShowImage1(wxCommandEvent& WXUNUSED(event)) {
    CamSel_ = Cam_;
    NumCamSel_ = 1;
  }
  void OnShowImage2(wxCommandEvent& WXUNUSED(event)) {
    CamSel_ = Cam2_;
    NumCamSel_ = 2;
  }
  void OnShowImage3(wxCommandEvent& WXUNUSED(event)) {
#ifdef BIAS_HAVE_PMD
    if(CamPMDShm_){
      CamSel_ = CamPMDShm_;
    }
    else
      CamSel_ = CamPMD_;
    NumCamSel_ = 3;
#endif
  }
  void OnShowImage4(wxCommandEvent& WXUNUSED(event)) {
#ifdef BIAS_HAVE_PMDUSB
    CamSel_ = CamPMDUSB_;
    NumCamSel_ = 4;
#endif
  }

  /**
      This is the main save method where the images are written do disk
  */
  void DoGrab_() {
    wxString text;
    if(Cam_!=NULL) {
      text.Printf(wxT("ShowCamWX-CAM0-%05d.mip"),GrabCounter_);
      ImageSaveThread *imsave;
      imsave = new ImageSaveThread(text, CamImg_);
      if ( imsave->Create() != wxTHREAD_NO_ERROR ) {
        BIASERR("Cannot create ImageSaveThread, saving unthreaded");
        ImageIO::Write(WxToAscii(text), CamImg_);
      }
      else {
        imsave->Run();
      }
    }
    if (Cam2_) {
      text.Printf(wxT("ShowCamWX-CAM1-%05d.mip"),GrabCounter_);
      ImageSaveThread *imsave2;
      imsave2 = new ImageSaveThread(text, CamImg2_);
      if ( imsave2->Create() != wxTHREAD_NO_ERROR ) {
        BIASERR("Cannot create ImageSaveThread, saving unthreaded");
        ImageIO::Write(WxToAscii(text), CamImg2_);
      }
      else {
        imsave2->Run();
      }
    }
#ifdef BIAS_HAVE_PMD
    if (CamPMD_ || CamPMDShm_) {
      text.Printf(wxT("ShowCamWX-PMD-%05d.tiff"),GrabCounter_);
      PMDImageMetaData meta;
      meta.width = 64;
      meta.height = 48;
      meta.channels = 1;
      meta.PrincipalX = 32;
      meta.PrincipalY = 24;
      meta.FocallengthX = 100;
      meta.AspectRatio = 1;
      meta.UndistCoeff[0] = 0;
      meta.UndistCoeff[1] = 0;
      meta.UndistCoeff[2] = 0;
      meta.UndistCoeff[3] = 0;  
      meta.CameraCenter[0] = 0;  
      meta.CameraCenter[1] = 0;  
      meta.CameraCenter[2] = 0;  
      meta.CameraRotation[0] = 0;  
      meta.CameraRotation[1] = 0;  
      meta.CameraRotation[2] = 0; 

      PMDImageProc::Write(WxToAscii(text), CamImgPMD_, 
                        CamImgPMDdep_, CamImgPMDamp_, 
                        meta, meta);
    }
#endif
#ifdef BIAS_HAVE_PMDUSB
    if (CamPMDUSB_) {
      text.Printf(wxT("ShowCamWX-PMDUSB-grey-%05d"),GrabCounter_);
      ImageIO::Write(WxToAscii(text), CamImgPMDUSB_);
      cout <<"Saved image: "<<text<<endl;
      text.Printf(wxT("ShowCamWX-PMDUSB-depth-%05d"),GrabCounter_);
      ImageIO::Write(WxToAscii(text), CamImgPMDUSBdep_);
      cout <<"Saved image: "<<text<<endl;
      text.Printf(wxT("ShowCamWX-PMDUSB-modCoeff-%05d"),GrabCounter_);
      ImageIO::Write(WxToAscii(text), CamImgPMDUSBamp_);
    }
#endif
    cout <<"Saved image: "<<text<<endl;
    GrabCounter_++;

  }


  void OnGrab(wxCommandEvent& WXUNUSED(event))
  { 
    DoGrab_();
  }

  void OnTimer(wxTimerEvent & /*evt*/)
  {
    float fps;
    Timer_.Stop();
    StopWatch_.Stop();
    fps = 1000000.0 / StopWatch_.GetRealTime();
    StopWatch_.Reset();
    StopWatch_.Start();
    FrameCounter_++;
    wxString text;
    text.Printf(wxT("%05d %2.1f fps "),FrameCounter_,fps);
    SetStatusText(text, 1);
    //LocalStopWatch_.Reset();
    //LocalStopWatch_.Start();
#ifdef WIN32
#ifdef HAVE_TRIGGER_CLASSES
    if (trig_) {
      trigger.Fire();
    }
#endif
#endif
    if(Cam_ != NULL) Cam_->GrabSingle(CamImg_);
    //LocalStopWatch_.Stop();
    //logstr << "1st image: " << LocalStopWatch_.GetRealTime()/1000 << "ms" << endl;
    if (Cam2_){
    //LocalStopWatch_.Reset();
    //LocalStopWatch_.Start();
      Cam2_->GrabSingle(CamImg2_);
    //logstr << "2nd image: " << LocalStopWatch_.GetRealTime()/1000 << "ms" << endl;
    }
#ifdef BIAS_HAVE_PMD
    if (CamPMD_){
      (static_cast<VideoSource_DcamPMD*>(CamPMD_))->GrabSingle(CamImgPMD_);
      (static_cast<VideoSource_DcamPMD*>(CamPMD_))->GrabSingleDepth(CamImgPMDdep_);
      (static_cast<VideoSource_DcamPMD*>(CamPMD_))->GrabSingleModCoeff(CamImgPMDamp_);
    } 

    if(CamPMDShm_){
      (static_cast<VideoSource_ShmPMD*>(CamPMDShm_))->GrabSingle(CamImgPMD_);
      (static_cast<VideoSource_ShmPMD*>(CamPMDShm_))->GrabSingleDepth(CamImgPMDdep_);
      (static_cast<VideoSource_ShmPMD*>(CamPMDShm_))->GrabSingleModCoeff(CamImgPMDamp_);
    }
#endif
#ifdef BIAS_HAVE_PMDUSB
    if (CamPMDUSB_){
      (static_cast<VideoSource_usbPMD*>(CamPMDUSB_))->GrabSingle(CamImgPMDUSB_);
      (static_cast<VideoSource_usbPMD*>(CamPMDUSB_))->GrabSingleDepth(CamImgPMDUSBdep_);
      (static_cast<VideoSource_usbPMD*>(CamPMDUSB_))->GrabSingleModCoeff(CamImgPMDUSBamp_);
    } 
#endif

    if (redgreen_ && Cam2_) {
      static StereoRedGreen comb;
      static Camera<unsigned char> stereo;
      comb.Combine(CamImg_, CamImg2_, stereo);
      ShowImg_ = stereo;
    }  else {
      if (NumCamSel_==1) ShowImg_ = CamImg_;
	//ImageConvert::ToGrey(CamImg_, ShowImg_);
      if (NumCamSel_==2)  ShowImg_ = CamImg2_;
      if (NumCamSel_==3) ShowImg_ = CamImgPMD_;
     // if (NumCamSel_==4) ShowImg_ = CamImgPMDUSB_;
	  if (NumCamSel_==4) {
		  BIAS::ImageConvert::ConvertST(CamImgPMDUSB_,ShowImg_,
			  BIAS::ImageBase::ST_unsignedchar);	  
	  }
    }

#ifdef BIAS_HAVE_DSHOW
    if( (dynamic_cast<VideoSource_DSHOW*>(CamSel_)) != NULL)
    {
      ShowImg_.Flip();
    }
#endif
    IC_->Show(ShowImg_,"foo"); 
    unsigned int w = ShowImg_.GetWidth();
    unsigned int h = ShowImg_.GetHeight();
    text.Printf(wxT("size: %4dx%4d"),w, h);
    SetStatusText(text, 2);

    if (w != lastwidth || h != lastheight) {
      lastwidth = w; lastheight = h;
      IC_->SetSize(w,h);
      Fit();
      cout <<"Refit"<<endl;
    }

    if(bSaveImages_){
      DoGrab_();
    }
    Timer_.Start(delay_,true);
  }   


  void OnGrabContinuous(wxCommandEvent& WXUNUSED(event))
  {    
    FrameCounter_ = 0;    
    bSaveImages_ = !bSaveImages_;
    cout<<"Saving images= "<<bSaveImages_;
  }

  void OnConfig(wxCommandEvent& WXUNUSED(event))
  {
#ifdef BIAS_HAVE_DSHOW
    if(Cam_!=NULL)
    {
      if((dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_)) != NULL)
        (dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_))->DisplayCameraProperties();
    }
#endif
  }

  void OnConfigPin(wxCommandEvent& WXUNUSED(event))
  {
#ifdef BIAS_HAVE_DSHOW
    if(Cam_!=NULL)
    {
      if((dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_)) != NULL)
        (dynamic_cast<BIAS::VideoSource_DSHOW *> (Cam_))->DisplaySourcePinProperties();
    }
#endif
  }
  void OnShowController(wxCommandEvent& WXUNUSED(event))
  {
    if (VideoController_) VideoController_->Show(true);
  }

 void OnShowController2(wxCommandEvent& WXUNUSED(event))
  {
    if (VideoController2_) VideoController2_->Show(true);
  }

 void OnShowControllerPMDUSB(wxCommandEvent& WXUNUSED(event))
  {
	  if (VideoControllerPMDUSB_){
		wxMessageDialog 
		dlg(NULL,wxT("Shutter is integration time for 3D part of the MultiCam\n"
		"Gain is shutter for 2D part of MutliCam! \n"));
		dlg.ShowModal();
		VideoControllerPMDUSB_->Show(true);
	  }
  }

protected:
  wxSlider *sliderGain;
  wxSlider *sliderShutter;
  VideoSource_Controller *VideoController_;
  VideoSource_Controller *VideoController2_;
  VideoSource_Controller *VideoControllerPMDUSB_;
  VideoSource *Cam_;
  VideoSource *Cam2_;
#ifdef WIN32
#ifdef HAVE_TRIGGER_CLASSES
  C1394CameraControlTrigger trigger1394;
	TriggerIOControl trigger;
#endif
#endif
  bool trig_;
#ifdef BIAS_HAVE_PMD
  VideoSource *CamPMD_;
  VideoSource *CamPMDShm_;
#endif
#ifdef BIAS_HAVE_PMDUSB
  VideoSource *CamPMDUSB_;
#endif
  VideoSource *CamSel_; // camera that is currently displayed 
  int NumCamSel_; // Number of this camera (PMD=3)
  ImageCanvas  *IC_;
  wxStatusBar *StatusBar_;
  TimeMeasure StopWatch_; 
  //TimeMeasure LocalStopWatch_; 
  bool redgreen_;
  unsigned int FrameCounter_;
  unsigned int GrabCounter_;
  unsigned int lastwidth,lastheight;
  Camera<unsigned char> CamImg_, CamImg2_, ShowImg_, CamImgPMD_, CamImgPMDUSB_;
#ifdef BIAS_HAVE_PMD
  Camera<float> CamImgPMDdep_, CamImgPMDamp_;
#endif
#ifdef BIAS_HAVE_PMDUSB
  Camera<float> CamImgPMDUSBdep_, CamImgPMDUSBamp_;
#endif
  wxTimer Timer_;

  //  VideoSource_Controller *MyControls_;

  DECLARE_EVENT_TABLE()

};



BEGIN_EVENT_TABLE(MyFrame, wxFrame)
EVT_MENU(id_quit, MyFrame::OnQuit)
EVT_MENU(id_config, MyFrame::OnConfig)
EVT_MENU(id_pin, MyFrame::OnConfigPin)
EVT_MENU(id_save, MyFrame::OnGrab)
EVT_TIMER(id_timer, MyFrame::OnTimer)
EVT_KEY_DOWN(MyFrame::OnKey)
EVT_MENU(id_save2, MyFrame::OnGrabContinuous) 
EVT_MENU(id_show1, MyFrame::OnShowImage1) 
EVT_MENU(id_show2, MyFrame::OnShowImage2) 
EVT_MENU(id_show3, MyFrame::OnShowImage3) 
EVT_MENU(id_show4, MyFrame::OnShowImage4) 
EVT_MENU(id_controller, MyFrame::OnShowController)
EVT_MENU(id_controller2, MyFrame::OnShowController2)
EVT_MENU(id_controllerPMDUSB, MyFrame::OnShowControllerPMDUSB)
END_EVENT_TABLE()  









// ================= MyApp =========================

class MyApp: public wxApp
{
public:
  virtual bool OnInit();
  ~MyApp();
  MyFrame *ToplevelWindow_;
};


void Usage()
{
  wxMessageDialog 
    usage(NULL,wxT("Usage: \n"
    "ShowCamWX -n | -N ip-string -P port |-v|-d|-k|-K bit-file [-x width] [-y height] [-i <string>] [-f fps]\n "
    "  -n: VideoSource_Net (UDP)\n"
    "  -N string (ip addr.): VideoSource_Net (TCP)\n"
    "         -P int (port): VideoSource_Net (TCP)\n"
    "  -m: VideoSource_Shm\n"
    "  -k: VideoSource_PMD\n"
    "  -K: VideoSource_PMDUSB\n"
    "  -s: string (Shm-name) : VideoSource_SHMPMD\n"
    "  -D: string (WDM device name) VideoSource_DSHOW\n"
    "  -d: VideoSource_DCam\n"
    "  -j: VideoSource_DV\n"
    "  -u: VideoSource_Ueye, only available if Windows compiled and with Option BUILD_UEYE\n" 
    "  -p: Portnumber to listen to\n"
    "  -v: VideoSource_V4L\n"
    "  -i string : device/interface\n"
                   "  -x int : image width\n" 
                   "  -y int : image height\n"
    "  -I string : 2nd device/interface"));
  usage.ShowModal();

  exit(0);
}

bool MyApp::OnInit()
{
  ConsoleRedirectIO();

  logstr.open("ShowCamWx.log");
  logstr <<"Start"<<endl;
  VideoSource *Cam=NULL;
  VideoSource *Cam2=NULL;
  VideoSource *CamPMD=NULL;
  VideoSource *CamPMDShm=NULL;
  VideoSource *CamPMDUSB=NULL;
  bool dv,dcam,net,v4l,TCP,shm,dshow,ueye,pmd,pmdusb,trig,shmpmd;
  dv=dcam=v4l=net=TCP=shm=dshow=ueye=pmd=pmdusb=shmpmd=trig=false;
  unsigned int w=0,h=0;
  unsigned int port=6002;
  float fps=0;
  int c=0;
  string device,ip_addr;
  string device2;
  string shmName;
  string bitFileName;

  WxToAsciiArray argvAscii(argv, argc);

  while ((c = getopt(argc, argvAscii, "nN:P:ms:dtkK:vx:y:f:i:I:D:uj")) != -1)
  {
    switch (c){
    case 'd': dcam = true ;break;
    case 'm': shm = true;break;
    case 's': shmpmd = true;shmName = optarg;break;
    case 'n': net = true;break;
    case 'N': net = true;TCP=true;ip_addr=optarg;break;
    case 'P': port=atoi(optarg);break;
    case 'v': v4l = true;break;
    case 'x': w=atoi(optarg); break;
    case 'y': h=atoi(optarg); break;
    case 'f': fps=atof(optarg); break;
    case 'i': device = optarg; break;
    case 'I': device2 = optarg; break;
    case 'p': port = atoi(optarg); break;
#ifdef WIN32
#ifdef HAVE_TRIGGER_CLASSES
    case 't': trig = true; break;
#endif
#endif
    case 'u':
#ifdef BIAS_HAVE_UEYE
      ueye =true;
#else
      Usage();
#endif
  break;
    case 'D': device = optarg; dshow = true; 
#ifndef WIN32
      Usage();
#endif
#ifndef BIAS_HAVE_DSHOW
      Usage();
#endif
      break;
#ifdef BIAS_HAVE_DV
    case 'j' : dv=true;
#else
      Usage();
#endif
      break;
#ifdef BIAS_HAVE_PMD
    case 'k': 
      logstr<<"using pmd\n";
      pmd = true ;      
#else
      Usage();
#endif
      break;
#ifdef BIAS_HAVE_PMDUSB
    case 'K': 
      logstr<<"using usbPMD\n";
      pmdusb = true ;
      bitFileName = optarg;
      break;
#endif
    case '?': 
    case 'h': 
    case ':': 
      Usage(); break;

    default:
      break;
    }
  }
  logstr <<"getopt done"<<endl;

  if (dcam && Cam==NULL) {
#ifdef BIAS_HAVE_DCAM
    VideoSource_DCAM *vsd = new VideoSource_DCAM;
    vsd->SetDebugLevel(D_DCAM);
    vsd->SetSize(1024,768,2);
    Cam=vsd;
    if (device2 != ""){
      Cam2 = new VideoSource_DCAM;
      Cam2->SetDebugLevel(D_DCAM);
    }
#ifdef WIN32
#ifdef HAVE_TRIGGER_CLASSES
    if (trig) {
      (static_cast<VideoSource_DCAM*>(Cam))->ExternalTrigger(true);
      if (Cam2) (static_cast<VideoSource_DCAM*>(Cam2))->ExternalTrigger(true);
    } else {
      (static_cast<VideoSource_DCAM*>(Cam))->ExternalTrigger(false);
      if (Cam2) (static_cast<VideoSource_DCAM*>(Cam2))->ExternalTrigger(false);
    }
#endif
#endif

#else
    cerr <<"DCam support disabled during compilation of BIAS"<<endl;
    exit(1);
#endif
  }//endif (dcam && Cam==NULL) 

#ifdef BIAS_HAVE_PMD
  if (pmd && CamPMD==NULL) {
    logstr <<"VS_DcamPMD"<<endl;
    VideoSource_DcamPMD *vsd = new VideoSource_DcamPMD;
    //vsd->SetDebugLevel(D_DCAM);
    CamPMD=vsd;
  }

  if(shmpmd && CamPMDShm==NULL)
  {
    logstr <<"VS_DcamPMDShm"<<endl;
    VideoSource_ShmPMD *vsd = new VideoSource_ShmPMD;
    //vsd->SetDebugLevel(D_DCAM);
    CamPMDShm=vsd;
  }
#endif
#ifdef BIAS_HAVE_PMDUSB
  if (pmdusb && CamPMDUSB==NULL) {
    logstr <<"VS_usbPMD"<<endl;
    VideoSource_usbPMD *vspu = new VideoSource_usbPMD;
    vspu->SetBitFile(bitFileName);
    //vsd->SetDebugLevel(D_DCAM);
    CamPMDUSB=vspu;
  }
#endif

  if (net && Cam==NULL) {
    logstr <<"VS_NET"<<endl;
    VideoSource_Net *vsn = new VideoSource_Net;
    vsn->OpenDevice( ip_addr,port);
    //      vsn->SetUseTCP(TCP);
    Cam=vsn;
  }

  if (shm && Cam==NULL) {
    logstr <<"VS_SHM"<<endl;
    VideoSource_Shm *vsshm = new VideoSource_Shm;
    Cam=vsshm;
    if (device2 != ""){
      Cam2 = new VideoSource_Shm;
    }
  }


  if (v4l && Cam==NULL) {
#ifdef BIAS_HAVE_V4L
    logstr <<"VS_V4L"<<endl;
    VideoSource_V4L *vsv4l = new VideoSource_V4L;
    Cam=vsv4l;
#else
    exit(1);
#endif
  }

#ifdef WIN32
#ifdef BIAS_HAVE_DSHOW
  if(dshow && Cam==NULL) {
    logstr <<"VS_DSHOW "<<endl;
    VideoSource_DSHOW *dsh = new VideoSource_DSHOW(false);
    //dsh->SetSizeAndColorModel(320,240,3,BIAS::ImageBase::CM_YUV420P);
    Cam=dsh;
  }
#endif
#endif //WIN32

#ifdef BIAS_HAVE_UEYE
  if(ueye && Cam==NULL) {
    logstr <<"VS_UEYE "<<endl;
    VideoSource_uEye *eye = new VideoSource_uEye;
    eye->SetSize(w, h);
    eye->SetLeftTop(56,0);
    eye->SetShutter(0.01f);
    eye->SetGain(0);

    Cam=eye;
  }
#endif // BIAS_HAVE_UEYE

#ifdef BIAS_HAVE_DV
  if(dv && Cam==NULL) {
    logstr <<"VS_DV "<<endl;
    VideoSource_DV  *dvcam = new VideoSource_DV;
    dvcam->SetDebugLevel(D_DV);
    Cam = dvcam;
  }
#endif // BIAS_HAVE_DV

  if(Cam != NULL){
    if (fps>0) {
      Cam->SetFPS(fps);
      if (Cam2!= NULL) Cam2->SetFPS(fps);
    }
    if (!ueye && !dshow){
      Cam->SetSize(w,h);
      if (Cam2!= NULL) Cam2->SetSize(w,h);
    }
  }
  else if(CamPMD==NULL && CamPMDShm==NULL && CamPMDUSB==NULL){
    Usage();
  } 
  //following cases are confusing: unwind possible?
  if (dcam && Cam != NULL) {
    Cam->SetColorModel(ImageBase::CM_YUV411);
    Cam->SetSize(w, h);
    if (device != ""){ 
#ifdef WIN32
      if (Cam->OpenDevice(0)<0) {
#else
      if (Cam->OpenDevice(atoi(device.c_str()))<0) {
#endif
        logstr << "Unable to find camera with ID:" << device.c_str() << endl;
        BIASABORT;
      } else {
        logstr <<"We have a specified device"<<endl;
      }
    } // device == "" following:
    else {
      if(Cam->OpenDevice()<0) {
        logstr <<"default camera could not be opened"<<endl;	
        BIASABORT;
      }
      logstr <<"No device specified, opened default"<<endl;	
    }

    if (Cam2!=NULL && device2 != ""){
#ifdef WIN32
      if (Cam2->OpenDevice(1)<0) {
#else
      if (Cam2->OpenDevice(atoi(device2.c_str()))<0) {
#endif
        logstr << "Unable to find 2nd camera with ID:" << device.c_str() << endl;
        BIASABORT;
      } else {
        logstr <<"We have a specified device"<<endl;
      }
    } else if (Cam2) {
      BIASERR("something funny happened");
      BIASABORT;
    }
  } // endif (dcam && Cam != NULL)
  else { 
    if (Cam!=NULL && device != ""){ 
      Cam->OpenDevice(device.c_str());
      logstr <<"We have a specified device"<<endl;
    }
    else{
      if(Cam!=NULL)
      {
        Cam->OpenDevice();
        logstr <<"No device specified, opened default"<<endl;	
      }
    }
    if (device2 != "" && Cam2 != NULL){ 
      Cam2->OpenDevice(device2.c_str());
      logstr <<"We have a specified device"<<endl;
    } else if (Cam2) {
      BIASERR("something funny happened");
      BIASABORT;
    }
  }


#ifdef BIAS_HAVE_PMD
  if (CamPMD != NULL){
    logstr <<"opening pmd device"<<endl;
	((VideoSource_DcamPMD*)CamPMD)->SetIntegrationTime(20000);
	  if (CamPMD->OpenDevice()<0) {
	    BIASERR("Failed to initialize PMD-Camera");
      BIASABORT;
    } else {
      logstr<<"great!"<<endl;
    }
  }
  else if(CamPMDShm != NULL){
    logstr <<"opening pmdshm device"<<endl;
    if ( ((VideoSource_ShmPMD*)CamPMDShm)->OpenDevice(shmName.c_str())<0) {
	    BIASERR("Failed to open Shm PMD-Camera");
      BIASABORT;
    } else {
      logstr<<"opened Shm PMD-Camera!"<<endl;
    }
  }
#endif
#ifdef BIAS_HAVE_PMDUSB
  if (CamPMDUSB != NULL){
    logstr <<"opening usb-pmd device"<<endl;
	  if (CamPMDUSB->OpenDevice()<0) {
	    BIASERR("Failed to initialize usbPMD-Camera");
      BIASABORT;
    } else {
      logstr<<"great!"<<endl;
    }
    ((VideoSource_usbPMD*)CamPMDUSB)->SetShutter(0.05f);
  }
#endif
  cout << "--------------------------------PREGRAB" << endl;
  if(Cam != NULL) Cam->PreGrab();
  if (Cam2 != NULL) Cam2->PreGrab();
  
#ifdef BIAS_HAVE_PMD
  if (CamPMD!= NULL) CamPMD->PreGrab();
  if (CamPMDShm != NULL) CamPMDShm->PreGrab();
#endif
#ifdef BIAS_HAVE_PMDUSB
  if (CamPMDUSB!= NULL) CamPMDUSB->PreGrab();
#endif

  ToplevelWindow_ = new MyFrame(wxT("ShowCamWX"),wxPoint(50,50),
    wxSize(400,340),Cam, Cam2, CamPMD, CamPMDShm, CamPMDUSB,trig);

  //   char *Server;
  //   if (VideoSource_Net *vsn = dynamic_cast<VideoSource_Net *>(Cam)) 
  //   {
  //      Server = vsn->WaitForServerAddr();
  // 	 logstr <<"VS_Server is"<<Server<<endl;
  //      cout <<"Server is: "<<Server<<endl;
  //   }


  if(Cam != NULL) Cam->SetDebugLevel(0);
  if (Cam2!= NULL) Cam2->SetDebugLevel(0);
  if (CamPMD!= NULL) CamPMD->SetDebugLevel(0);
  if (CamPMDShm!= NULL) CamPMDShm->SetDebugLevel(0);
  if (CamPMDUSB!= NULL) CamPMDUSB->SetDebugLevel(0);
  return TRUE;
}

MyApp::~MyApp()
{  
}


IMPLEMENT_APP(MyApp)
