#include "CylindricDepthPanorama.hh"
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>
#include "ThreeDOut.hh"

using namespace BIAS;
using namespace std;

/** Constructor */
CylindricDepthPanorama::CylindricDepthPanorama(double rangePhi,
                                               double rangeHeight,
                                               int samplingPhi,
                                               int samplingHeight){
  //Init the panorama image here
  panorama_.Init(int(rangePhi*samplingPhi)*2,
                 int(rangeHeight*samplingHeight)*2,1);
  panoramaHitCount_.Init(int(rangePhi*samplingPhi)*2,
                         int(rangeHeight*samplingHeight)*2,1);
  panoramaTexture_.Init(int(rangePhi*samplingPhi)*2,
                        int(rangeHeight*samplingHeight)*2,1);

  panoramaHitCount_.FillImageWithConstValue(0.0);
  panoramaWidth_ = (unsigned int)(rangePhi*samplingPhi)*2;
  panoramaHeight_ =(unsigned int)(rangeHeight*samplingHeight)*2;
  samplingPhi_ = samplingPhi;
  samplingHeight_ = samplingHeight;
  rangePhi_ = rangePhi;
  rangeHeight_ = rangeHeight;
  pInitialProjection_ = NULL;
  pCylinderProj_ = NULL; 
  bTextureFilled_ = false;

}

/** Destructor */
CylindricDepthPanorama::~CylindricDepthPanorama(){
  if( pInitialProjection_ != NULL){
    delete pInitialProjection_;
    pInitialProjection_ = NULL;
  }
  if(pCylinderProj_){
    delete pCylinderProj_;
    pCylinderProj_ = NULL;
  }
}


int  CylindricDepthPanorama::
ProcessCylindricProjection(BIAS::Image<float> &depthMap,
                           BIAS::Projection & p,
                           BIAS::Image<unsigned char> *texture){

  int ret =0;
  float **ida = depthMap.GetImageDataArray();
  float **idaP = panorama_.GetImageDataArray();
  float **idaPHC = panoramaHitCount_.GetImageDataArray();
  unsigned char ** idaTex = panoramaTexture_.GetImageDataArray();

  int width = depthMap.GetWidth();
  int height = depthMap.GetHeight();
 
  float depthValue=0.0;
  BIAS::HomgPoint3D point3D;  
  BIAS::HomgPoint2D point2D,cylPoint;
  BIAS::Vector3<double> ray3D;

  //Create the cylindrical Projection from initial Projection
  if(pInitialProjection_ == NULL){        
    pInitialProjection_ = new BIAS::Projection(p);    
    pCylinderProj_ = 
      new ProjectionParametersCylindric(-tan(0.58),tan(0.58),
                                        -M_PI,M_PI, 
                                        int(rangeHeight_*samplingHeight_),
                                        int(rangePhi_*samplingPhi_));
                                                         
    //contruct cylindrical projection      
    pCylinderProj_->SetImageSize(int(rangePhi_*samplingPhi_),
                                 int(rangeHeight_*samplingHeight_));     
    pCylinderProj_->SetQC(pInitialProjection_->GetQ(),
                          pInitialProjection_->GetC());
    pCylinderProj_->SetPrincipal(int(rangePhi_*samplingPhi_)/2,
                                 int(rangeHeight_*samplingHeight_)/2);  
   //  const Vector3< double > axis(0,0,1);
//     pCylinderProj_->SetCylinder(axis,M_PI/2,BIAS::Vector3<double>(0,0,0));
    pCylinderProj_->Print();      
  }
  
 
  unsigned int x1,y1;
  for(int x=0;x<width;x++){
    for(int y=0;y<height;y++){      
      point2D[0]=x;
      point2D[1]=y;
      point2D[2]=0;            
      depthValue = ida[y][x];   
      if (depthValue != 0.0) {       
        // Unproject to a 3Dpoint
        point3D = p.UnProjectToPoint(point2D,depthValue);        
        cylPoint = pCylinderProj_->Project(point3D);
        x1 = (unsigned int)cylPoint[0];
        y1 = (unsigned int)cylPoint[1];       
  
        
        //   cout<<"3DPoint: "<<point3D<<" Cylindercoords: "<<cylPoint<<endl;
                         
        if(y1>=0.0 && y1<panoramaHeight_ && 
           x1>=0.0 && x1<panoramaWidth_) {
          //fill texture panorama with values
          if(texture != NULL){
            idaTex[y1][x1] = texture->GetImageDataArray()[y][x];         
            bTextureFilled_ = true;
          }
          if(idaPHC[y1][x1] == 0.0){
            idaP[y1][x1]=depthValue;
            idaPHC[y1][x1]=1.0;
          }
          else{          
            idaP[y1][x1]= 
              (idaP[y1][x1] * idaPHC[y1][x1]+depthValue)/(idaPHC[y1][x1]+1);
            idaPHC[y1][x1]+=1.0;
          }
        }
      }
    }
  }
  return ret;



}


int CylindricDepthPanorama::
Process(BIAS::Image<float> &depthMap,
               BIAS::Projection & p){


  int ret =0;
  float **ida = depthMap.GetImageDataArray();
  float **idaP = panorama_.GetImageDataArray();
  float **idaPHC = panoramaHitCount_.GetImageDataArray();
  double yCoord=0.0,xCoord=0.0;
  int width = depthMap.GetWidth();
  int height = depthMap.GetHeight();
  //   unsigned int projWidth,projHeight;
  //   double pX, pY;
  float depthValue=0.0;
  BIAS::HomgPoint3D point3D;  
  BIAS::HomgPoint2D point2D,cylPoint;
  BIAS::Vector3<double> ray3D;
  //determine the projection of the centerpoint of first image
  // then form rest around it
  if(pInitialProjection_ == NULL){
     
    depthValue = ida[int(height/2.0)][int(width/2.0)];    
    if(depthValue!=0.0){
      
      pInitialProjection_ = new BIAS::Projection(p);
      //contruct cylindrical projection
      pCylinderProj_->
        SetCylinder(ProjectionParametersCylindric::AXIS_ORTHOGONAL);
      pCylinderProj_->SetImageSize(int(rangePhi_*samplingPhi_),
                                   int(rangeHeight_*samplingHeight_));     
      pCylinderProj_->SetQC(pInitialProjection_->GetQ(),
                            pInitialProjection_->GetC());
    
      //Get the point in the middle
      point2D[0] = width/2.0;
      point2D[1] = height/2.0;
      point2D[2] = 0;      
      point3D=pInitialProjection_->UnProjectToPoint(point2D,depthValue);
      //    cylPoint = pCylinderProj_->Project(point3D);
//       double xCoord = (int)cylPoint[0];
//       double yCoord = (int)cylPoint[1];       

      //       xCoord = xCoord*(180/M_PI)*samplingPhi_;
      //       yCoord = yCoord*(180/M_PI)*samplingHeight_;

      //      point3D = pInitialProjection_->UnProjectToRay(point2D);
      //get x and y of first center projection  
      xCoord = point3D[0];      
      yCoord = point3D[1];
      

      xCoord = (asin(xCoord/depthValue))*(180/M_PI)*samplingPhi_;   
      yCoord = (asin(yCoord/depthValue))*(180/M_PI)*samplingHeight_;
   
      dOffsetX_ = (int)(panoramaWidth_/2 + xCoord);
      dOffsetY_ = (int)(panoramaHeight_/2 + yCoord);       
      cout<<"Initial Offset: "<<dOffsetX_<<","<<dOffsetY_<<endl;
    }
  }
 
  unsigned int x1,y1;
  for(int x=0;x<width;x++){
    for(int y=0;y<height;y++){      
      point2D[0]=x;
      point2D[1]=y;
      point2D[2]=0;            
      depthValue = ida[y][x];   
      if (depthValue != 0.0) {       
        // Unproject to a 3Dpoint
        point3D = p.UnProjectToPoint(point2D,depthValue);        
        //    cylPoint = pCylinderProj_->Project(point3D);
        // xCoord = (int)cylPoint[0];
//         yCoord = (int)cylPoint[1];       
        
        //xCoord = xCoord*(180/M_PI)*samplingPhi_;
        // yCoord = yCoord*(180/M_PI)*samplingHeight_;
        
        //  cout<<"3DPoint: "<<point3D<<" Cylindercoords: "<<cylPoint<<endl;
        //the points in space     
        xCoord = point3D[0];                
        yCoord = point3D[1];     
         
        xCoord = (asin(xCoord/depthValue))*(180/M_PI)*samplingPhi_+dOffsetX_;
        yCoord = (asin(yCoord/depthValue))*(180/M_PI)*samplingHeight_+dOffsetY_;
           
        x1 = (unsigned int)xCoord;
        y1 = (unsigned int)yCoord;

        if(y1>=0.0 && y1<panoramaHeight_ && 
           x1>=0.0 && x1<panoramaWidth_) {
          
          if(idaPHC[y1][x1] == 0.0){
            idaP[y1][x1]=depthValue;
            idaPHC[y1][x1]=1.0;
          }
          else{          
            idaP[y1][x1]= 
              (idaP[y1][x1] * idaPHC[y1][x1]+depthValue)/(idaPHC[y1][x1]+1);
            idaPHC[y1][x1]+=1.0;
          }
        }
      }
    }
  }
  return ret;

}

int  CylindricDepthPanorama::
GetPanorama(BIAS::Image<float> &image, 
            BIAS::Image<unsigned char> &texture){
  int ret =0;
  image = panorama_; 
  if(bTextureFilled_){
    texture = panoramaTexture_;
  }
  else{
    BIAS::ImageConvert::ConvertST(panorama_,texture,
                                  BIAS::ImageBase::ST_unsignedchar);
    texture.ScaleShiftBetween(0,255);
  }
  return ret;
}


int CylindricDepthPanorama::
GetTriangleMesh(BIAS::TriangleMesh &mesh){
  int ret =-1; 
  BIAS::Image<unsigned char> tmpUC;
  if(bTextureFilled_){
    if(pCylinderProj_ != NULL){
      BIAS::Projection proj(*pCylinderProj_);
      mesh.GenerateDenseMesh(panorama_,proj,panoramaTexture_,0,0,
                             panoramaWidth_,panoramaHeight_);

      return 0;
    }  
    else
      return ret;

  }
  else{
    BIAS::ImageConvert::ConvertST(panorama_,tmpUC,
                                  BIAS::ImageBase::ST_unsignedchar);
    tmpUC.ScaleShiftBetween(0,255);
    if(pCylinderProj_ != NULL){
      BIAS::Projection proj(*pCylinderProj_);
      mesh.GenerateDenseMesh(panorama_,proj,tmpUC,0,0,
                             panoramaWidth_,panoramaHeight_);

      return 0;
    }
    else
      return ret;
    
  } 
}
