#ifndef _StatekeeperGL__hh__
#define _StatekeeperGL__hh__

#include <bias_config.h>

#ifndef BIAS_HAVE_OPENGL
#  error Please recompile BIAS with USE_OPENGL
#endif // BIAS_HAVE_OPENGL


#  include <Gui/biasgl.h>

namespace BIAS {

  /** push and pop GL states on constructor/destructor 
  Helper class pop(save) OpenGL states on construction
  and push them on destruction automatically.
  Using matchiung  push/pop pairs is easy with this class. 
  Sample usage:
  \verbatim
  // begin var. scop
  {
  // save state by push
  StatekeeperGL myTmp;   
  // change the state, fire GL calls 
  ...   
  // reload state by destructors pop call automatically
  }
  // end var. scope
  \endverbatim
  There should not be a performance impact if you use display lists. 
  @author Jan Woetzel */
  class BIASUtils_EXPORT StatekeeperGL {
  public:
    /// constructor, pushes major OpenGL states
    StatekeeperGL(){
      glPushAttrib( 0
        //|GL_ALL_ATTRIB_BITS 
        |GL_ENABLE_BIT 
        |GL_COLOR_BUFFER_BIT 
        |GL_CURRENT_BIT 
        //|GL_DEPTH_BUFFER_BIT 
        //|GL_LIGHTING_BIT 
        //|GL_SCISSOR_BIT 
        //|GL_STENCIL_BUFFER_BIT 
        );
    };

    /** constructor, pushes OpenGL state as given by mask
    GLbitfield mask */
    StatekeeperGL(const GLbitfield & mask){
      glPushAttrib(mask);
    }

    /// destructor, pops OpenGL state 
    ~StatekeeperGL(){
      glPopAttrib();
    }

  }; // end of class
} // end namespace


#endif

