
#include <Image/Camera.hh>
#include <Base/Image/ImageConvert.hh>
#include <Base/Image/ImageIO.hh>
#include <VideoSource/VideoSource_DCAM.hh>
#include <Gui/GuiGTK.hh>
#include <unistd.h>
#include <getopt.h>


using namespace BIAS;
using namespace std;


#ifdef BUILD_IMAGE_USHORT
#define StorageType unsigned char

#include <Base/Debug/TimeMeasure.hh>

float rgain=-31.0/32.0, bgain=31.0/32.0;
float gain = 0.0;
unsigned savecounter=0;


void Get8bitsfrom16(Image<unsigned short> &src, Image<unsigned char> &dst,
		    unsigned int shift = 0)
{
  unsigned short *us = src.GetImageData();
  unsigned char *uc = dst.GetImageData();
  unsigned int pc = src.GetPixelCount() * src.GetChannelCount();
  int t;
  for (unsigned int i=0;i<pc; i++) {
    t = ((us[i])>>shift);
    if (t> 255) uc[i] = 255;
    else uc[i] = t & 255;
  }
}






void mainloop(VideoSource_DCAM *cam)
{
  GuiGTK CamWin;
  bool done = false;
  char c;
  char title[100];
  
  struct timeval now;
  float fps;
  int i,savedframes;
  unsigned int shift = 2;
  long start=0, end;
  bool color, pause, autosave=false;
  ostringstream os;
  
  TimeMeasure t1,t2;

  Camera<unsigned char>  pic1;
  Camera <unsigned short> pic;
  sprintf(title,"MyWindow");
  CamWin.SetTitle(title);
  cam->InitImage(pic);
  ImageIO::Write("foo",pic);

  pic1.Init(pic.GetWidth(), pic.GetHeight(),1);

  pause = false;
  color = true;

  i = 1;
  savedframes = 0;
  
  bool save;
  //pic2.SetDebugLevel(D_CONVERT);
  
  while (!done){ 
    save = false;
    gettimeofday(&now, NULL);
    end = start;
    start =  (now.tv_sec % 1000) * 1000 + (now.tv_usec / 1000);
    fps = 1.0 / (float)(start-end) * 1000.0 ;

    c = CamWin.CheckForKeyEvent();
    switch (c) {
    case 'q': 
    case 27: 
      done = true;
      break;
    case 'g': 
      cam->AutoMode(FEATURE_GAIN, true);
      cerr << "\nauto gain on"<<endl;
      break;
    case '1': 
      cout <<endl<<"Shutter: "<<cam->GetShutter()<<endl;
//       cam->SetShutter(0.01);
      break;
    case 'w': 
      cam->OnePushAuto(FEATURE_WHITE_BALANCE);
      cerr << "\nwhite balance"<<endl;
      break; 
    case 'S':
    case ' ':
      save = true;
      break;
    case 's':
      autosave=!autosave;
      if (autosave) 
        cerr << "\nstarted saving sequence at frame "<<i<<" into frame"
             <<setfill('0')<<setw(5)<<savecounter<<".mip"<<endl;
      else 
        cerr << "\nstopped saving sequence at frame "<<i<<" into frame"
             <<setfill('0')<<setw(5)<<savecounter<<".mip"<<endl;
      break;
    case 'p': 
      pause = !pause;	  
      break;
    case 'c': 
      color = !color;
      if (color) cerr << "\nshowing color image" << endl;
      else cerr << "\nshowing grey image" << endl;
      break;
    case 'G': 
      cam->AutoMode(FEATURE_GAIN, false);
      cerr << "\nauto gain off"<<endl;
      break;
    case 'e':
      if (shift >0) shift--;
      cout <<endl<<"shift: "<<shift<<endl;
      break;
    case 'E':
      if (shift <4) shift++;
      cout <<endl<<"shift: "<<shift<<endl;
      break;
    case '[':
      gain -= 1.0/32.0;
      cam->SetGain(gain);
      cout<<"Overall gain :"<<gain<<endl;
      break;
    case ']':
      gain += 1.0/32.0;
      cam->SetGain(gain);
      cout<<"Overall gain :"<<gain<<endl;
      break;
    default:
      if (c!=-1) cerr <<"unknown key "<<c<<endl;
      break;
    }

            
    if (!pause) {
      cam->GrabSingle(pic);

      unsigned short min, max;
      pic.GetMinMaxPixelValue(min, max);
      cout << "min: "<<min<<"\tmax: "<<max<<"  "<<endl;

      Get8bitsfrom16(pic,pic1,shift);
//       pic1.SetColorModel(pic.GetColorModel());
      pic1.SetColorModel(ImageBase::CM_Bayer_RGGB);

      
      if (save) {
        os.str("");
        os << "frame" << setfill('0') << setw(5) << savecounter;
        ImageIO::Write(os.str(), pic, true);
// 	ImageIO::ExportImage(os.str(),pic, ImageIO::FF_ppm);
      cerr << "\nsaved frame "<<i<<" in "<<os.str()<<endl;
	os <<"_uc";
	ImageIO::Write(os.str(), pic1, true);
        save = false;
        savecounter++;
      }
      if (autosave){
        os.str("");
        os << "frame" << setfill('0') << setw(5) << savecounter;
        if (ImageIO::Write(os.str(), pic, true)!=0){
          BIASERR("error saving frame "<<i<<" at file "<<os.str()<<".mip");
        }
        savecounter++;
      } 

       CamWin.ShowImage(pic1);

       cout <<"Frame: "<<i<<"  Showing  fps: "<<fps<<"\r";
       cout.flush();
	  
      i++;
    } // if (!pause) 
  }  
}







int main(int argc, char *argv[])
{
  int res;

  VideoSource_DCAM *cam;

  // prepare camera
  cam = new VideoSource_DCAM;

  // cam->SetSize(1280,960,2.0);
  //cam->SetSize(1600,1200,2);
//   cam->SetSize(1024,768,2);
   //  cam->SetSize(320,250,2);
  cam->SetBytesPerPixel(2);

  cam->SetDebugLevel(D_DCAM |D_DCAM_F7);
  res = cam->OpenDevice() ;
  cout <<"Openresult: "<<res<<endl;
  
  if (res <0) {
    cerr<<" Can not open camera"<<endl;
    exit(1);
  }

  cam->PreGrab();

  cout <<" g  : Auto Gain ON\n";
  cout <<" G  : Auto Gain OFF\n";
  cout <<"123 : Gamma mode"<<endl;
  cout <<" w  : OnePushWhitebalance (not for PointGrey DragonFly)\n";
  cout <<" 7/8: decr/incr blue gain (for white balancing manual)\n";
  cout <<" 4/5: decr/incr red gain (for white balancing manual)\n";
  cout <<" c  : Color/Grey \n";
  cout <<"SPC : save an image \n";
  cout <<" s  : save sequence\n";
  cout <<" p  : Stop/Resume\n";
  cout <<" e  : toggle between fast and normal conversion\n";
  cout <<" q  : Quit      \n";

  cam->SetDebugLevel(0);
//   cam->SetFeature(FEATURE_SHARPNESS,0);

  mainloop(cam );
  
  cam->PostGrab();

  cam->CloseDevice();
  delete cam;
  
  return 0;
}

#else
int main(int argc, char *argv[]) 
{
    BIASERR("showdcam16 can only work if BIAS was compiled with BUILD_IMAGE_USHORT");
}
#endif

