/* 
This file is part of the BIAS library (Basic ImageAlgorithmS).

Copyright (C) 2003, 2004    (see file CONTACTS for details)
  Multimediale Systeme der Informationsverarbeitung
  Institut fuer Informatik
  Christian-Albrechts-Universitaet Kiel


BIAS is free software; you can redistribute it and/or modify
it under the terms of the GNU Lesser General Public License as published by
the Free Software Foundation; either version 2.1 of the License, or
(at your option) any later version.

BIAS is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License
along with BIAS; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __MIPVIDEOSOURCE_DCAM_HH__
#define __MIPVIDEOSOURCE_DCAM_HH__

#include <bias_config.h>


#if defined(WIN32) && !defined(BIAS_HAVE_1394CAMERACMU) 
#  error You need 1394CAMERACMU for BIAS Videosource_DCAM. Please recompile BIAS with USE_1394CAMERACMU.
#endif 

#if !defined(WIN32) && !defined(BIAS_HAVE_DC1394) 
#  error You need DC1394 for BIAS Videosource_DCAM. Please recompile BIAS with USE_DC1394.
#endif 

// consistency check, DCAM is set if one of theabove is set
#if !defined(BIAS_HAVE_1394CAMERACMU) && !defined(BIAS_HAVE_DC1394) 
#  error You need either DC1394 or 1394CameraCMU  for BIAS Videosource_DCAM. Please recompile BIAS with one of them.
#endif 




#include <Base/Common/W32Compat.hh>

#ifndef WIN32
#  include <sys/time.h>
#endif

#include <Image/Camera.hh>

#ifdef WIN32
#  include <1394Camera.h>
typedef unsigned long long int u_int64_t;
#else
#  include <libdc1394/dc1394_control.h>
#endif
#include <VideoSource/VideoSource_Base.hh>

 
// only this number of cards are probed
#ifndef MAX_1394_CARDS
#define MAX_1394_CARDS 2
#endif


#define MAX_1394_SOURCES 16

#ifndef MAX_1394_CHANNELS
#define MAX_1394_CHANNELS 256
#endif

#define MAX_DMABUFFERS 2



typedef struct CameraParameter_s
{
  // these are the real parameters
  float Shutter;
  float Gain;
  unsigned int WhiteBalance;
  bool AutoShutter;
  bool AutoGain;
  bool AutoWhiteBalance;
  unsigned char Sharpen;
  unsigned char GammaMode;
  unsigned char OpticalFilter;
  

  // these bools describe active changes
  bool ShutterChanged;
  bool GainChanged;
  bool WhiteBalanceChanged;
  bool AutoShutterChanged;
  bool AutoGainChanged;
  bool AutoWhiteBalanceChanged;
  bool SharpenChanged;
  bool GammaModeChanged;
  bool OpticalFilterChanged;
  
  // one-push  action
  bool WhiteBalanceOnePush;
  

} CameraParameterSet;





/// support function to get the number of cameras and IDs of all cameras on one bus
extern int GetCamIDs(int busid ,int &ncams, int *&camids);



namespace BIAS {

  /** 
      \class VideoSource_DCAM
      \ingroup VideoG
      \author evers

      \brief This class VideoSource_DCAM implements access to IEEE1394 (Firewire, iLink) camers following the DCam 1.2 standard protocoll.

  */
  typedef enum { 
    CAMERA_SONY_DFW_X700  = 0, // IEEE1394
    CAMERA_SONY_DFW_V500,// IEEE1394
    CAMERA_PTGREY_DRAGONFLY_COLOR ,// IEEE1394
    CAMERA_PTGREY_DRAGONFLY_HIRES ,// IEEE1394
    CAMERA_PTGREY_DRAGONFLY_BW ,// IEEE1394
    CAMERA_PTGREY_DRAGONFLY2_COLOR_HIRES,
    CAMERA_PTGREY_SCORPION_COLOR_1600 ,// IEEE1394
    CAMERA_PTGREY_FIREFLY_COLOR, //IEEE1394
    CAMERA_PTGREY_FIREFLY_BW, //IEEE1394
    CAMERA_SONY_EVID31, // analog via frame grabber, PTZ-camera
    CAMERA_SONY_DV,
    CAMERA_UNIBRAIN_FIREI,
    CAMERA_UNKNOWN 
  } CameraModel;
  
  typedef enum {
    LENS_SONY_EVID31_BUILTIN,
    LENS_SONY_DV_BUILTIN,
    LENS_PENTAX_CMOUNT_4_8,
    LENS_PENTAX_CMOUNT_8_0,
    LENS_PENTAX_CMOUNT_4_2,
    LENS_UNKNOWN
  } LensModel;

#ifdef WIN32

  // only define these values in windows.
  // in linux they are defined in dc1394_control.h (NOT in namespace BIAS)

  typedef enum {
    FEATURE_BRIGHTNESS,
    FEATURE_EXPOSURE,
    FEATURE_SHARPNESS,
    FEATURE_WHITE_BALANCE,
    FEATURE_HUE,
    FEATURE_SATURATION,
    FEATURE_GAMMA,
    FEATURE_SHUTTER,
    FEATURE_GAIN,
    FEATURE_IRIS,
    FEATURE_FOCUS,
    FEATURE_TEMPERATURE,
    FEATURE_TRIGGER,

    FEATURE_ZOOM,
    FEATURE_PAN,
    FEATURE_TILT,
    FEATURE_OPTICAL_FILTER,

    FEATURE_CAPTURE_SIZE,
    FEATURE_CAPTURE_QUALITY
  };


#define FEATURE_MIN   FEATURE_BRIGHTNESS
#define FEATURE_MAX   FEATURE_CAPTURE_QUALITY
#define NUM_FEATURES (FEATURE_MAX - FEATURE_MIN + 1)
#endif // WIN32


  class BIASVideoSource_EXPORT VideoSource_DCAM : public VideoSource{ 

  public: 
    /*! Activates the timestamp for PointGrey Scorpion SCOR-20SO
     */
    void ActivateScorpionTimestamp();
    bool ScorpionTimestampActive_;

    VideoSource_DCAM(int Card = 0);
    ~VideoSource_DCAM();


    int OpenDevice();
    int OpenDevice(int camid);
    /** this a special function to by-pass all automatic initializations and 
	Use only in conjuction with SertSize() and SetLeftTop() !
    */
    int SetFormat7(int mode, int bpp, int colorchannels);
      void SetCam_ID(int manualcamid_); 
    int CloseDevice();
    // evil hack, fixme, please fixme !!! (I forgot the TODO)
    void SetBinning(bool b) {UseBinning_ = b;};
  
    /// Select the port (bus-id) to use, only valid before OpenDevice()
    int UsePort(int channel);
  
    int GrabSingle(BIAS::Camera <unsigned char> &image);

#ifdef BUILD_IMAGE_USHORT
    int GrabSingle(BIAS::Camera <unsigned short> &image);
#endif  
  
    int PreGrab();
  
    int PostGrab();
  
    int ShowCapabilities();

    void SetAutoShutter(bool b);
    bool GetAutoShutter();
    
    void SetAutoGain(bool b);
    bool GetAutoGain();

    // these 3 are only for DCAM, nothing else.
    int AutoMode( int feature, bool on);
    int  SetFeature( int feature, unsigned int value);
    unsigned int GetFeature( int feature);

    int OnePushAuto(int feature);
    bool OnePushRunning(int feature);
  
    int SetShutter(float exptime);
    float GetShutter();
    int SetGain(float g);
    float GetGain();

    int SetWhiteBalance(float rvalue,float bvalue);

    int SetGammaMode(unsigned char g);
    unsigned char  GetGammaMode();
    
    int ExternalTrigger(bool on);
  
    // format7 specific methods
    inline unsigned int GetTop(){return Top_;};
    inline unsigned int GetLeft(){return Left_;};

             
    /// set the topleft corner for partitial scan
    int SetLeftTop(unsigned int left, unsigned  int top);

    inline int GetTransmissionTime(){return TransmissionTime_;};
  
    /// set camera to parameters specified in complete set
    int SetCamera(CameraParameterSet camparam);

    virtual void SetCameraID(int /*id*/){BIASERR("Can not set camera id for DCams, they are identified automatically");};

    // this choses f7 mode even for standard resolution
    void ForceFormat7(bool forcef7) {ForceF7_ =forcef7;}

    void SetBytesPerPacket(const int b){BytesPerPacket_ = b;}
int UserSizeToMode_8bit_();  int SelectDefault();
  protected:

    // part of OpenDevice()
    int OpenBus_();
  
    int InitCam();
    int StartIso(); 
    int StopIso(); 
    int QueryCam();

    // main dispatcher
    int UserSizeToMode();

    // for all formats with 16bit per pixel per channel (aka short int)
    int UserSizeToMode_16bit_();

    // for all formats with standard 8bpp per channel (unsigned char image)
      // int UserSizeToMode_8bit_();
    int UserSizeToFormat7_();
    void PrintFormatAndMode_();
      //  int SelectDefault();
    /** 
        Set this value and read back what really happend. */
    int ApplyBytesPerPacket();

    /** Evaluate Min and Max BpP, framesize and fps, then calc BpP */
    unsigned int CalcBytesPerPacket();

    /// use some magic to determine vendor and model


    

    // there is only one handle per bus for multiple devices 
#ifdef WIN32
    C1394Camera camera;  
    CameraModel GetModelForEuid(LARGE_INTEGER euid);
#else // WIN32
    static raw1394handle_t IEEE1394Handle_[MAX_1394_CARDS];
    static int Instances_[MAX_1394_CARDS];
    static int InstancesTotal_;
    static int BytesPerCyle_[MAX_1394_CARDS];

    CameraModel GetModelForEuid(const  u_int64_t  euid);
      int camid;
    // this is the card to serve for
    int IEEE1394Card_;
  
    dc1394_cameracapture DcamCapture_;
    dc1394_camerainfo DcamInfo_;
    dc1394_feature_set DcamFeatures_;

    nodeid_t * DcamNodes_;

    // which modes are supported by this cam ?
    bool SupportedFormats_[NUM_FORMATS];

    // list all Modes for each format
    bool SupportedModes_[NUM_FORMATS][NUM_FORMAT1_MODES];

    int SupportedFramerates_[NUM_FORMATS][NUM_FORMAT1_MODES];
    bool SupportedFeature_[NUM_FEATURES];

#endif // WIN32

    int NumCams_;
    int SelectedCam_;
    int DcamMode_;
    int DcamFormat_;
    int DcamFps_;
    int DcamIsoChannel_;
    int Trigger_;

    bool ForceF7_;
    bool DirectF7_;
    bool UseDMA_;
    int DcamDMABuffers_;

    // used to set Format 7 Scalabel Image Size
    unsigned int MaxWidth_,  MaxHeight_, Left_,Top_;
    unsigned int UnitWidth_, UnitHeight_;
    bool UserTopLeft_;
    int BytesPerPacket_;
    unsigned int MaxBpP_, MinBpP_;

    // time it lasts to transmit one image in usec
    int TransmissionTime_;


    unsigned char Shift16_; // uses this to shift bits down to normal
    CameraModel CamModel_;
    LensModel Lens_;
    bool UseBinning_;

  };


} // namespace MIP

#endif // __MIPVIDEOSOURCE_DCAM_HH__
