#ifndef __BIAS_VIDEOSOURCE_NET_HH__
#define __BIAS_VIDEOSOURCE_NET_HH__

#include <bias_config.h>
#include <Base/Common/BIASpragmaStart.hh>

#include <iostream>
#include <sstream>
#include <VideoSource/VideoSource_Base.hh>

#include <NetworkComm/CScommClient.hh>



#ifdef BIAS_HAVE_LIBJPEG
# include <Image/CompressJpeg.hh>
#endif //BIAS_HAVE_LIBJPEG

namespace BIAS{
  /**\class VideoSource_Net
     \ingroup VideoG
     \brief This class implements a video streaming client using TCP.
     Use VideoServer to feed this beast.
     @author grest, Nov. 2005
  */
  class BIASVideoSource_EXPORT VideoSource_Net : public VideoSource {
  
  public:   
    /** Standard Constructor
        use threaded param for dis-/enabling threaded receiving!
            If threaded receiving is disabled, keepAll-param will
            be enabled for client_!
            (you'll get every image that was sent)
    */    
    VideoSource_Net(bool threaded = true);
    
    virtual ~VideoSource_Net() { delete pImgBuffer_; }
  
  
    /** use given port and ip address to connect to videoserver*/
    virtual int OpenDevice(const char* serverName);
    
    /** use given port and ip address to connect to videoserver*/
    virtual int OpenDevice(const std::string &serverName,
                           int port=D_CS_DEFAULT_PORT);

    /** for UDP commm, listen on this port for data */
    virtual int OpenDevice(int port=D_CS_DEFAULT_PORT);
    
    int PreGrab();
    virtual int GrabSingle(BIAS::Camera <unsigned char> &image);
    virtual int GrabSingle(BIAS::ImageBase &image);
    int PostGrab();

    int CloseDevice();

  protected:  
    std::stringstream *pImgBuffer_; // pointer needed to free mem
    int count_;
    bool UseTCP_;

    int port_;
    CScommClient client_;


#ifdef BIAS_HAVE_LIBJPEG
    BIAS::Image<unsigned char> jpegImg_;
    JpegHandler Decompressor_;
#endif //BIAS_HAVE_LIBJPEG
  };


} // namespace BIAS


#include <Base/Common/BIASpragmaEnd.hh>

#endif

