#!/bin/sh -x
# -create a source package for distribution
#
# USAGE
# place your prompt within the BIAS source dir
# set CMAKE_MODULE_PATH env. var to CMakeModuless source dir
#
# cmake has no "make dist" target, 
# this script substitute it.
# NOTES
#
# AUTHOR
# Jan Friso Evers 2004-06-03
# Jan Woetzel 2006


# remove obviously obsolete stuff
if [ -f Makefile ]; then
 make clean 
 make distclean
fi

#default
VERSION="0.0.0"
#extract version info as fild content after quote in first line with token
MAJOR=`grep BIAS_VERSION_MAJOR CMakeLists.txt |head -n1 |cut -d \" -f2`
MINOR=`grep BIAS_VERSION_MINOR CMakeLists.txt |head -n1 |cut -d \" -f2`
PATCH=`grep BIAS_VERSION_PATCH CMakeLists.txt |head -n1 |cut -d \" -f2`
VERSION="$MAJOR.$MINOR.$PATCH"
if [ -n "$1" ]; then 
  VERSION="$1";
fi
echo VERSION=$VERSION

SUBDIR="BIAS-$VERSION"


# cleanup old fragments
if [ -d $SUBDIR ]; then
 rm -rf $SUBDIR
fi

pwd

# put everything into $SUBDIR
echo  SUBDIR: $SUBDIR
mkdir -p $SUBDIR

# copy CMakeModules 
mkdir -p $SUBDIR/CMakeModules/
echo CMAKE_MODULE_PATH=$CMAKE_MODULE_PATH
if [ -r "$CMAKE_MODULE_PATH" ] ; then
  echo copying from CMAKE_MODULE_PATH=$CMAKE_MODULE_PATH
  cp $CMAKE_MODULE_PATH/Define*.cmake $SUBDIR/CMakeModules/
  cp $CMAKE_MODULE_PATH/Find*.cmake   $SUBDIR/CMakeModules/
  cp $CMAKE_MODULE_PATH/Use*.cmake    $SUBDIR/CMakeModules/
  cp $CMAKE_MODULE_PATH/Target*.cmake $SUBDIR/CMakeModules/
  cp $CMAKE_MODULE_PATH/Test*.cmake   $SUBDIR/CMakeModules/
  cp $CMAKE_MODULE_PATH/*.cpp         $SUBDIR/CMakeModules/
  cp $CMAKE_MODULE_PATH/*.in          $SUBDIR/CMakeModules/
else 
 echo ERROR: please set CMAKE_MODULE_PATH
 exit
fi


#cp CMakeLists.txt CMakeLists.txt.orig
# DEPRECATED: patching not rewuired 
# patch CMakeLists.txt to use proper CMAKE_MODULE_PATH
#sed -e s/'SET(CMAKE_MODULE_PATH $ENV{CMAKE_MODULE_PATH})'/'SET(CMAKE_MODULE_PATH ${BIAS_SOURCE_DIR}\/CMakeModules)'/g CMakeLists.txt > CMakeLists.txt.patched
#cp CMakeLists.txt.patched  CMakeLists.txt


# remove unwanted files
rm $SUBDIR/make_dist.sh # $SUBDIR/configure 
rm -rf $SUBDIR/Doc/html
# delete unused and configured files:
rm -f DOXYGEN.test bias_config.h bias-config CMakeCache.txt BIASBuildSettings.cmake BIASConfig.cmake BIASLibraryDepends.cmake DartTestfile.txt DartConfiguration.tcl cmake_install.cmake 
rm -f *.sln  *.vcproj.* *.vcproj *.suo *.ncb 

pwd 

# clean old versions to avoid recursive packing
rm -f BIAS*.tgz
rm -f BIAS*.tar
rm -f BIAS*.tar.gz

# put everything into $SUBDIR
tar c * |tar -xC $SUBDIR --exclude Testing \
--exclude=$SUBDIR \
--exclude=Makefile \
--exclude=makefile \
--exclude=CVS \
--exclude=\*~ \
--exclude=\#*~ \
--exclude=\*.o \
--exclude=\*.so \
--exclude=\*.a \
--exclude=\*.obj \
--exclude=\*.lib \
--exclude=\*.dll \
--exclude=\*.pdb \
--exclude=\*.exe \
--exclude=\*.sln \
--exclude=\*.vcproj \
--exclude=\*.vcproj.*\
--exclude=\*.sln \
--exclude=\*.suo \
--exclude=\*.ncb 

# DEPRECATED
# undo patching of CMAKE_MODULE_PATH
# cp -f CMakeLists.txt.orig CMakeLists.txt

# do not create tgz in current dir because 
# to avoid recursive packaging.
OUTNAME="BIAS-$VERSION"

tar czf ${OUTNAME}.tar.gz $SUBDIR
chmod 755  ${OUTNAME}.tar.gz

# cleanup
#rm -rf $SUBDIR
